@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -22,10 +22,9 @@
 
 #pragma once
 
-#include "ai/lua/aspect_advancements.hpp"
 #include "attack_prediction.hpp"
-#include "units/alignment.hpp"
 #include "units/ptr.hpp"
+#include "units/unit_alignments.hpp"
 
 #include <vector>
 
@@ -60,10 +59,6 @@ struct battle_context_unit_stats
 	bool petrifies;          /**< Attack petrifies opponent when it hits. */
 	bool plagues;            /**< Attack turns opponent into a zombie when fatal. */
 	bool poisons;            /**< Attack poisons opponent when it hits. */
-	bool backstab_pos;       /**<
-	                           * True if the attacker is in *position* to backstab the defender (this is used to
-                               * determine whether to apply the backstab bonus in case the attacker has backstab).
-                               */
 	bool swarm;              /**< Attack has swarm special. */
 	bool firststrike;        /**< Attack has firststrike special. */
 	bool disable;            /**< Attack has disable special. */
@@ -90,8 +85,7 @@ struct battle_context_unit_stats
 			bool attacking,
 			nonempty_unit_const_ptr opp,
 			const map_location& opp_loc,
-			const_attack_ptr opp_weapon,
-			const unit_map& units);
+			const_attack_ptr opp_weapon);
 
 	/** Used by AI for combat analysis, and by statistics_dialog */
 	battle_context_unit_stats(const unit_type* u_type,
@@ -138,7 +132,6 @@ struct battle_context_unit_stats
 		, petrifies(false)
 		, plagues(false)
 		, poisons(false)
-		, backstab_pos(false)
 		, swarm(do_swarm)
 		, firststrike(first)
 		, disable(false)
@@ -231,12 +224,10 @@ private:
 			int attacker_weapon,
 			nonempty_unit_const_ptr defender,
 			const map_location& defender_loc,
-			int defender_weapon,
-			const unit_map& units);
+			int defender_weapon);
 
 	static battle_context choose_attacker_weapon(nonempty_unit_const_ptr attacker,
 			nonempty_unit_const_ptr defender,
-			const unit_map& units,
 			const map_location& attacker_loc,
 			const map_location& defender_loc,
 			double harm_weight,
@@ -245,7 +236,6 @@ private:
 	static battle_context choose_defender_weapon(nonempty_unit_const_ptr attacker,
 			nonempty_unit_const_ptr defender,
 			unsigned attacker_weapon,
-			const unit_map& units,
 			const map_location& attacker_loc,
 			const map_location& defender_loc,
 			const combatant* prev_def);
@@ -288,7 +278,7 @@ int under_leadership(const unit &u, const map_location& loc, const_attack_ptr we
 int combat_modifier(const unit_map& units,
 		const gamemap& map,
 		const map_location& loc,
-		UNIT_ALIGNMENT alignment,
+		unit_alignments::type alignment,
 		bool is_fearless);
 
 /**
@@ -296,14 +286,14 @@ int combat_modifier(const unit_map& units,
  * due to the current time of day.
  */
 int combat_modifier(const time_of_day& effective_tod,
-		UNIT_ALIGNMENT alignment,
+		unit_alignments::type alignment,
 		bool is_fearless);
 
 /**
  * Returns the amount that a unit's damage should be multiplied by
  * due to a given lawful_bonus.
  */
-int generic_combat_modifier(int lawful_bonus, UNIT_ALIGNMENT alignment, bool is_fearless, int max_liminal_bonus);
+int generic_combat_modifier(int lawful_bonus, unit_alignments::type alignment, bool is_fearless, int max_liminal_bonus);
 /**
  * Function to check if an attack will satisfy the requirements for backstab.
  * Input:
