@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by Iris Morelle <shadowm2006@gmail.com>
 	Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
@@ -218,6 +218,16 @@ public:
 		return conn_ && conn_->using_tls();
 	}
 
+	const std::string& server_id() const
+	{
+		return server_id_;
+	}
+
+	const std::string& server_version() const
+	{
+		return server_version_;
+	}
+
 private:
 	enum class transfer_mode {download, connect, upload};
 
