@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2011 - 2022
+	Copyright (C) 2011 - 2024
 	by Dmitry Kovalenko <nephro.wes@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -21,7 +21,8 @@
 #pragma once
 
 #include "config.hpp"
-#include "lua/lua.h"
+#include "log.hpp"
+#include "lua/wrapper_lua.h"
 #include "map/location.hpp"
 #include "resources.hpp"
 #include "scripting/lua_common.hpp"
@@ -34,6 +35,9 @@
 #include <string>
 #include <vector>
 
+static lg::log_domain log_scripting_lua("scripting/lua");
+#define ERR_OBJ_LUA LOG_STREAM(err, log_scripting_lua)
+
 namespace ai {
 
 class lua_object_base {
@@ -286,11 +290,21 @@ inline std::shared_ptr<std::vector<target> > lua_object< std::vector<target> >::
 
 		lua_pushstring(L, "type"); // st n + 2
 		lua_rawget(L, -2);  // st n + 2
-		target::TYPE type = target::TYPE::EXPLICIT;
+		std::optional<ai_target::type> type = ai_target::type::xplicit;
 		if(lua_isnumber(L, -1)) {
-			type = target::TYPE::from_int(static_cast<int>(lua_tointeger(L, -1)));  // st n + 2
+			int target = static_cast<int>(lua_tointeger(L, -1));
+			type = ai_target::get_enum(target);  // st n + 2
+			if(!type) {
+				ERR_OBJ_LUA << "Failed to convert ai target type of " << target << ", skipping.";
+				continue;
+			}
 		} else if(lua_isstring(L, -1)) {
-			type = target::TYPE::string_to_enum(lua_tostring(L, -1));  // st n + 2
+			std::string target = lua_tostring(L, -1);
+			type = ai_target::get_enum(target);  // st n + 2
+			if(!type) {
+				ERR_OBJ_LUA << "Failed to convert ai target type of " << target << ", skipping.";
+				continue;
+			}
 		}
 		lua_pop(L, 1); // st n + 1
 
@@ -300,7 +314,7 @@ inline std::shared_ptr<std::vector<target> > lua_object< std::vector<target> >::
 
 		map_location ml(x, y, wml_loc());
 
-		targets->emplace_back(ml, value, type);
+		targets->emplace_back(ml, value, *type);
 	}
 
 	lua_settop(L, n);
