@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -18,7 +18,7 @@
 #include "preferences/game.hpp"
 #include "gettext.hpp"
 
-#include "video.hpp" //CVideo::get_singleton().window_state()
+#include "video.hpp" // window_has_flags()
 
 #ifdef HAVE_LIBDBUS
 #include "desktop/dbus_features.hpp"
@@ -56,14 +56,9 @@ bool available()
 
 void send(const std::string& owner, const std::string& message, type t)
 {
-	CVideo& video = CVideo::get_singleton();
-
-	// Do not show notifications when the window is visible...
-	if(video.window_has_flags(SDL_WINDOW_SHOWN)) {
-		// ... and it has a focus.
-		if(video.window_has_flags(SDL_WINDOW_MOUSE_FOCUS | SDL_WINDOW_INPUT_FOCUS)) {
-			return;
-		}
+	// Do not show notifications when the window is visible and has focus
+	if(video::window_is_visible() && video::window_has_focus()) {
+		return;
 	}
 
 #ifdef HAVE_LIBDBUS
