@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by Tomasz Sniatowski <kailoran@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -15,6 +15,7 @@
 
 #define GETTEXT_DOMAIN "wesnoth-editor"
 
+#include "draw.hpp"
 #include "editor/controller/editor_controller.hpp"
 #include "editor/editor_display.hpp"
 #include "floating_label.hpp"
@@ -25,6 +26,7 @@
 #include "team.hpp"
 #include "terrain/builder.hpp"
 #include "units/map.hpp"
+#include "video.hpp"
 
 namespace wb {
 	class manager;
@@ -32,12 +34,12 @@ namespace wb {
 
 namespace editor {
 
-editor_display::editor_display(editor_controller& controller, reports& reports_object, const config& theme_cfg)
-	: display(nullptr, std::shared_ptr<wb::manager>(), reports_object, theme_cfg, config())
+editor_display::editor_display(editor_controller& controller, reports& reports_object)
+	: display(nullptr, std::shared_ptr<wb::manager>(), reports_object, "editor", config())
 	, brush_locations_()
 	, controller_(controller)
+	, mouseover_hex_overlay_()
 {
-	video().clear_screen();
 }
 
 void editor_display::add_brush_loc(const map_location& hex)
@@ -69,60 +71,47 @@ void editor_display::rebuild_terrain(const map_location &loc) {
 	builder_->rebuild_terrain(loc);
 }
 
-void editor_display::pre_draw()
-{
-	// If we're showing hexes near the north of the map, put the help string at the bottom of the screen.
-	// Otherwise, put it at the top.
-	if(help_handle_ != 0) {
-		const bool place_at_top = get_visible_hexes().top[0] > 2;
-
-		if(place_at_top != help_string_at_top_) {
-			const auto& r = font::get_floating_label_rect(help_handle_);
-			double delta = map_outside_area().h - r.h;
-			if(place_at_top) {
-				font::move_floating_label(help_handle_, 0.0, -delta);
-			} else {
-				font::move_floating_label(help_handle_, 0.0, delta);
-			}
-			help_string_at_top_ = place_at_top;
-		}
-	}
-}
-
-image::TYPE editor_display::get_image_type(const map_location& loc)
-{
-	if (map().in_selection(loc)) {
-		return image::BRIGHTENED;
-	}
-	return image::TOD_COLORED;
-}
-
 void editor_display::draw_hex(const map_location& loc)
 {
-	int xpos = get_location_x(loc);
-	int ypos = get_location_y(loc);
 	display::draw_hex(loc);
-	if (map().on_board_with_border(loc)) {
-		if (map().in_selection(loc)) {
-			drawing_buffer_add(LAYER_FOG_SHROUD, loc, xpos, ypos,
-				image::get_image("editor/selection-overlay.png", image::TOD_COLORED));
-		}
 
-		if (brush_locations_.find(loc) != brush_locations_.end()) {
-			static const image::locator brush(game_config::images::editor_brush);
-			drawing_buffer_add(LAYER_SELECTED_HEX, loc, xpos, ypos,
-					image::get_image(brush, image::SCALED_TO_HEX));
-		}
+	if(!map().on_board_with_border(loc) || map_screenshot_) {
+		return;
+	}
+
+	if(map().in_selection(loc)) {
+		drawing_buffer_add(LAYER_FOG_SHROUD, loc,
+			[tex = image::get_texture(image::locator{"editor/selection-overlay.png"}, image::TOD_COLORED)](const rect& d) {
+				draw::blit(tex, d);
+			});
+	}
+
+	if(brush_locations_.find(loc) != brush_locations_.end()) {
+		static const image::locator brush(game_config::images::editor_brush);
+		drawing_buffer_add(LAYER_SELECTED_HEX, loc, [tex = image::get_texture(brush, image::HEXED)](const rect& d) {
+			draw::blit(tex, d);
+		});
+	}
+
+	// Paint mouseover overlays
+	if(mouseover_hex_overlay_ && loc == mouseoverHex_) {
+		drawing_buffer_add(LAYER_MOUSEOVER_OVERLAY, loc, [this](const rect& dest) {
+			mouseover_hex_overlay_.set_alpha_mod(196);
+			draw::blit(mouseover_hex_overlay_, dest);
+			mouseover_hex_overlay_.set_alpha_mod(SDL_ALPHA_OPAQUE);
+		});
 	}
 }
 
-const SDL_Rect& editor_display::get_clip_rect()
+rect editor_display::get_clip_rect() const
 {
 	return map_outside_area();
 }
 
-void editor_display::draw_sidebar()
+void editor_display::layout()
 {
+	display::layout();
+
 	config element;
 	config::attribute_value &text = element.add_child("element")["text"];
 	// Fill in the terrain report
@@ -141,6 +130,23 @@ void editor_display::draw_sidebar()
 		refresh_report("villages");
 		refresh_report("num_units");
 	}
+
+	// If we're showing hexes near the north of the map, put the help string at the bottom of the screen.
+	// Otherwise, put it at the top.
+	if(help_handle_ != 0) {
+		const bool place_at_top = get_visible_hexes().top[0] > 2;
+
+		if(place_at_top != help_string_at_top_) {
+			const auto& r = font::get_floating_label_rect(help_handle_);
+			double delta = map_outside_area().h - r.h;
+			if(place_at_top) {
+				font::move_floating_label(help_handle_, 0.0, -delta);
+			} else {
+				font::move_floating_label(help_handle_, 0.0, delta);
+			}
+			help_string_at_top_ = place_at_top;
+		}
+	}
 }
 
 const time_of_day& editor_display::get_time_of_day(const map_location& /*loc*/) const
@@ -166,9 +172,10 @@ void editor_display::set_help_string(const std::string& str)
 	const color_t color{0, 0, 0, 0xbb};
 
 	int size = font::SIZE_LARGE;
+	point canvas_size = video::game_canvas_size();
 
 	while(size > 0) {
-		if(font::pango_line_width(str, size) * 2 > video().get_width()) {
+		if(font::pango_line_width(str, size) * 2 > canvas_size.x) {
 			size--;
 		} else {
 			break;
@@ -179,13 +186,13 @@ void editor_display::set_help_string(const std::string& str)
 
 	font::floating_label flabel(str);
 	flabel.set_font_size(size);
-	flabel.set_position(video().get_width() / 2, video().get_height());
+	flabel.set_position(canvas_size.x / 2, canvas_size.y);
 	flabel.set_bg_color(color);
 	flabel.set_border_size(border);
 
 	help_handle_ = font::add_floating_label(flabel);
 
-	// Put the label near the bottom of the screen. In pre_draw it'll be moved to the top if the
+	// Put the label near the bottom of the screen. In layout() it'll be moved to the top if the
 	// user is editing hexes at the south edge of the map.
 	help_string_at_top_ = false;
 	const auto& r = font::get_floating_label_rect(help_handle_);
