@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2021
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -18,6 +18,8 @@
 #include "resources.hpp"
 #include "team.hpp"
 
+#include "addon/validation.hpp"
+
 #include "display.hpp"
 #include "editor/map/context_manager.hpp"
 #include "editor/map/map_context.hpp"
@@ -27,12 +29,14 @@
 #include "generators/map_create.hpp"
 #include "generators/map_generator.hpp"
 #include "gettext.hpp"
+#include "video.hpp"
 
 #include "editor/action/action.hpp"
 #include "editor/controller/editor_controller.hpp"
 #include "preferences/editor.hpp"
 
 #include "gui/dialogs/edit_text.hpp"
+#include "gui/dialogs/prompt.hpp"
 #include "gui/dialogs/editor/generate_map.hpp"
 #include "gui/dialogs/editor/new_map.hpp"
 #include "gui/dialogs/editor/resize_map.hpp"
@@ -43,11 +47,13 @@
 
 #include "gui/dialogs/editor/edit_scenario.hpp"
 #include "gui/dialogs/editor/edit_side.hpp"
+#include "gui/dialogs/editor/edit_pbl.hpp"
 #include "game_config_view.hpp"
 
 #include "terrain/translation.hpp"
 
 #include <memory>
+#include <boost/algorithm/string.hpp>
 
 namespace editor {
 
@@ -66,11 +72,12 @@ static const std::string get_menu_marker(const bool changed)
 	return ss.str();
 }
 
-context_manager::context_manager(editor_display& gui, const game_config_view& game_config)
+context_manager::context_manager(editor_display& gui, const game_config_view& game_config, const std::string& addon_id)
 	: locs_(nullptr)
 	, gui_(gui)
 	, game_config_(game_config)
 	, default_dir_(preferences::editor::default_dir())
+	, current_addon_(addon_id)
 	, map_generators_()
 	, last_map_generator_(nullptr)
 	, current_context_index_(0)
@@ -91,7 +98,7 @@ context_manager::context_manager(editor_display& gui, const game_config_view& ga
 context_manager::~context_manager()
 {
 	// Restore default window title
-	CVideo::get_singleton().set_window_title(game_config::get_default_title_string());
+	video::set_window_title(game_config::get_default_title_string());
 
 	resources::filter_con = nullptr;
 }
@@ -200,9 +207,9 @@ std::size_t context_manager::modified_maps(std::string& message)
 
 void context_manager::load_map_dialog(bool force_same_context /* = false */)
 {
-	std::string fn = filesystem::directory_name(get_map_context().get_filename());
+	std::string fn = get_map_context().get_filename();
 	if(fn.empty()) {
-		fn = default_dir_;
+		fn = filesystem::get_legacy_editor_dir()+"/maps";
 	}
 
 	gui2::dialogs::file_dialog dlg;
@@ -215,7 +222,7 @@ void context_manager::load_map_dialog(bool force_same_context /* = false */)
 	}
 }
 
-void context_manager::load_mru_item(unsigned int index, bool force_same_context /* = false */)
+void context_manager::load_mru_item(unsigned index, bool force_same_context /* = false */)
 {
 	const std::vector<std::string>& mru = preferences::editor::recent_files();
 	if(mru.empty() || index >= mru.size()) {
@@ -236,13 +243,41 @@ void context_manager::edit_side_dialog(int side_index)
 	}
 }
 
+void context_manager::edit_pbl()
+{
+	if(current_addon_ != "") {
+		std::string pbl = filesystem::get_current_editor_dir(current_addon_) + "/_server.pbl";
+		gui2::dialogs::editor_edit_pbl::execute(pbl, current_addon_);
+	}
+}
+
+void context_manager::change_addon_id()
+{
+	std::string new_addon_id = current_addon_;
+	gui2::dialogs::prompt::execute(new_addon_id);
+
+	if(addon_filename_legal(new_addon_id) && filesystem::rename_dir(filesystem::get_current_editor_dir(current_addon_), filesystem::get_current_editor_dir(new_addon_id))) {
+		std::string main_cfg = filesystem::get_current_editor_dir(new_addon_id)+"/_main.cfg";
+		std::string main = filesystem::read_file(main_cfg);
+
+		// update paths
+		boost::replace_all(main, "/"+current_addon_, "/"+new_addon_id);
+		// update textdomain
+		boost::replace_all(main, "wesnoth-"+current_addon_, "wesnoth-"+new_addon_id);
+		filesystem::write_file(main_cfg, main);
+
+		current_addon_ = new_addon_id;
+
+		for(context_ptr& context : map_contexts_) {
+			context->set_addon_id(current_addon_);
+		}
+	}
+}
+
 void context_manager::edit_scenario_dialog()
 {
 	map_context& context = get_map_context();
 
-	// TODO
-	//std::string fn = filesystem::directory_name(context.get_filename());
-
 	std::string id          = context.get_id();
 	std::string name        = context.get_name();
 	std::string description = context.get_description();
@@ -464,7 +499,7 @@ void context_manager::apply_mask_dialog()
 {
 	std::string fn = get_map_context().get_filename();
 	if(fn.empty()) {
-		fn = default_dir_;
+		fn = filesystem::get_dir(filesystem::get_current_editor_dir(current_addon_) + "/masks");
 	}
 
 	gui2::dialogs::file_dialog dlg;
@@ -474,7 +509,7 @@ void context_manager::apply_mask_dialog()
 
 	if(dlg.show()) {
 		try {
-			map_context mask(game_config_, dlg.path());
+			map_context mask(game_config_, dlg.path(), current_addon_);
 			editor_action_apply_mask a(mask.map());
 			perform_refresh(a);
 		} catch (const editor_map_load_exception& e) {
@@ -507,7 +542,7 @@ void context_manager::create_mask_to_dialog()
 {
 	std::string fn = get_map_context().get_filename();
 	if(fn.empty()) {
-		fn = default_dir_;
+		fn = filesystem::get_dir(filesystem::get_current_editor_dir(current_addon_) + "/masks");
 	}
 
 	gui2::dialogs::file_dialog dlg;
@@ -517,7 +552,7 @@ void context_manager::create_mask_to_dialog()
 
 	if(dlg.show()) {
 		try {
-			map_context map(game_config_, dlg.path());
+			map_context map(game_config_, dlg.path(), current_addon_);
 			editor_action_create_mask a(map.map());
 			perform_refresh(a);
 		} catch (const editor_map_load_exception& e) {
@@ -609,7 +644,7 @@ void context_manager::resize_map_dialog()
 				break;
 			default:
 				y_offset = 0;
-				WRN_ED << "Unknown resize expand direction" << std::endl;
+				WRN_ED << "Unknown resize expand direction";
 				break;
 		}
 
@@ -642,7 +677,7 @@ void context_manager::save_map_as_dialog()
 {
 	std::string input_name = get_map_context().get_filename();
 	if(input_name.empty()) {
-		input_name = filesystem::get_dir(default_dir_ + "/maps");
+		input_name = filesystem::get_legacy_editor_dir()+"/maps";
 	}
 
 	gui2::dialogs::file_dialog dlg;
@@ -656,14 +691,25 @@ void context_manager::save_map_as_dialog()
 		return;
 	}
 
-	save_map_as(dlg.path());
+	std::size_t is_open = check_open_map(dlg.path());
+	if(is_open < map_contexts_.size() && is_open != static_cast<unsigned>(current_context_index_)) {
+		gui2::show_transient_message(_("This map is already open."), dlg.path());
+	}
+
+	std::string old_filename = get_map_context().get_filename();
+
+	get_map_context().set_filename(dlg.path());
+
+	if(!write_map(true)) {
+		get_map_context().set_filename(old_filename);
+	}
 }
 
 void context_manager::save_scenario_as_dialog()
 {
 	std::string input_name = get_map_context().get_filename();
 	if(input_name.empty()) {
-		input_name = filesystem::get_dir(default_dir_ + "/scenarios");
+		input_name = filesystem::get_legacy_editor_dir()+"/scenarios";
 	}
 
 	gui2::dialogs::file_dialog dlg;
@@ -671,13 +717,27 @@ void context_manager::save_scenario_as_dialog()
 	dlg.set_title(_("Save Scenario As"))
 	   .set_save_mode(true)
 	   .set_path(input_name)
-	   .set_extension(".cfg");
+	   .set_extension(".cfg")
+	   .add_extra_path(desktop::GAME_EDITOR_MAP_DIR);
 
 	if(!dlg.show()) {
 		return;
 	}
 
-	save_scenario_as(dlg.path());
+	std::size_t is_open = check_open_map(dlg.path());
+	if(is_open < map_contexts_.size() && is_open != static_cast<unsigned>(current_context_index_)) {
+		gui2::show_transient_message(_("This scenario is already open."), dlg.path());
+		return;
+	}
+
+	std::string old_filename = get_map_context().get_filename();
+
+	get_map_context().set_filename(dlg.path());
+
+	if(!write_scenario(true)) {
+		get_map_context().set_filename(old_filename);
+		return;
+	}
 }
 
 void context_manager::init_map_generators(const game_config_view& game_config)
@@ -687,12 +747,12 @@ void context_manager::init_map_generators(const game_config_view& game_config)
 			continue;
 		}
 
-		const config& generator_cfg = i.child("generator");
-		if(!generator_cfg) {
-			ERR_ED << "Scenario \"" << i["name"] << "\" with id " << i["id"]
-					<< " has map_generation= but no [generator] tag" << std::endl;
+		// TODO: we should probably use `child` with a try/catch block once that function throws
+		if(const auto generator_cfg = i.optional_child("generator")) {
+			map_generators_.emplace_back(create_map_generator(i["map_generation"].empty() ? i["scenario_generation"] : i["map_generation"], generator_cfg.value()));
 		} else {
-			map_generators_.emplace_back(create_map_generator(i["map_generation"].empty() ? i["scenario_generation"] : i["map_generation"], generator_cfg));
+			ERR_ED << "Scenario \"" << i["name"] << "\" with id " << i["id"]
+					<< " has map_generation= but no [generator] tag";
 		}
 	}
 }
@@ -757,6 +817,11 @@ void context_manager::save_all_maps(bool auto_save_windows)
 			if(name.empty() || filesystem::is_directory(name)) {
 				std::ostringstream s;
 				s << default_dir_ << "/" << "window_" << i + 1;
+				if(!get_map_context().is_embedded() && !get_map_context().is_pure_map()) {
+					s << ".cfg";
+				} else {
+					s << ".map";
+				}
 				name = s.str();
 				get_map_context().set_filename(name);
 			}
@@ -786,52 +851,6 @@ void context_manager::save_map()
 	}
 }
 
-bool context_manager::save_scenario_as(const std::string& filename)
-{
-	std::size_t is_open = check_open_map(filename);
-	if(is_open < map_contexts_.size() && is_open != static_cast<unsigned>(current_context_index_)) {
-		gui2::show_transient_message(_("This scenario is already open."), filename);
-		return false;
-	}
-
-	std::string old_filename = get_map_context().get_filename();
-	bool embedded = get_map_context().is_embedded();
-
-	get_map_context().set_filename(filename);
-	get_map_context().set_embedded(false);
-
-	if(!write_scenario(true)) {
-		get_map_context().set_filename(old_filename);
-		get_map_context().set_embedded(embedded);
-		return false;
-	}
-
-	return true;
-}
-
-bool context_manager::save_map_as(const std::string& filename)
-{
-	std::size_t is_open = check_open_map(filename);
-	if(is_open < map_contexts_.size() && is_open != static_cast<unsigned>(current_context_index_)) {
-		gui2::show_transient_message(_("This map is already open."), filename);
-		return false;
-	}
-
-	std::string old_filename = get_map_context().get_filename();
-	bool embedded = get_map_context().is_embedded();
-
-	get_map_context().set_filename(filename);
-	get_map_context().set_embedded(false);
-
-	if(!write_map(true)) {
-		get_map_context().set_filename(old_filename);
-		get_map_context().set_embedded(embedded);
-		return false;
-	}
-
-	return true;
-}
-
 bool context_manager::write_scenario(bool display_confirmation)
 {
 	try {
@@ -890,10 +909,26 @@ void context_manager::load_map(const std::string& filename, bool new_context)
 		return;
 	}
 
-	LOG_ED << "Load map: " << filename << (new_context ? " (new)" : " (same)") << "\n";
+	if(filesystem::ends_with(filename, ".cfg")) {
+		if(editor_controller::current_addon_id_ == "") {
+			// if no addon id has been set and the file being loaded is from an addon
+			// then use the file path to determine the addon rather than showing a dialog
+			editor_controller::current_addon_id_ = filesystem::get_addon_id_from_path(filename);
+			if(editor_controller::current_addon_id_ == "") {
+				editor_controller::current_addon_id_ = editor::initialize_addon();
+			}
+			set_addon_id(editor_controller::current_addon_id_);
+		}
+
+		if(editor_controller::current_addon_id_ == "") {
+			return;
+		}
+	}
+
+	LOG_ED << "Load map: " << filename << (new_context ? " (new)" : " (same)");
 	try {
 		{
-			context_ptr mc(new map_context(game_config_, filename));
+			context_ptr mc(new map_context(game_config_, filename, current_addon_));
 			if(mc->get_filename() != filename) {
 				if(new_context && check_switch_open_map(mc->get_filename())) {
 					return;
@@ -913,20 +948,7 @@ void context_manager::load_map(const std::string& filename, bool new_context)
 			gui2::show_transient_message(_("Map loaded from scenario"), msg);
 		} else {
 			if(get_map_context().get_filename() != filename) {
-				if(get_map_context().get_map_data_key().empty()) {
-					ERR_ED << "Internal error, map context filename changed: "
-						<< filename << " -> " << get_map_context().get_filename()
-						<< " with no apparent scenario load\n";
-				} else {
-					utils::string_map symbols;
-					symbols["old"] = filename;
-					const std::string& msg = _("Loaded referenced map file:\n$new");
-					symbols["new"] = get_map_context().get_filename();
-					symbols["map_data"] = get_map_context().get_map_data_key();
-					gui2::show_transient_message(_("Map loaded from scenario"),
-						//TODO: msg is already translated does vgettext make sense?
-						VGETTEXT(msg.c_str(), symbols));
-				}
+				gui2::show_transient_message(_("Map loaded from scenario"), _("Loaded referenced map file:")+"\n"+get_map_context().get_filename());
 			}
 		}
 	} catch(const editor_map_load_exception& e) {
@@ -943,7 +965,7 @@ void context_manager::revert_map()
 
 	std::string filename = get_map_context().get_filename();
 	if(filename.empty()) {
-		ERR_ED << "Empty filename in map revert" << std::endl;
+		ERR_ED << "Empty filename in map revert";
 		return;
 	}
 
@@ -952,27 +974,27 @@ void context_manager::revert_map()
 
 void context_manager::new_map(int width, int height, const t_translation::terrain_code& fill, bool new_context)
 {
-	const config& default_schedule = game_config_.find_child("editor_times", "id", "empty");
+	const config& default_schedule = game_config_.find_mandatory_child("editor_times", "id", "empty");
 	editor_map m(width, height, fill);
 
 	if(new_context) {
-		int new_id = add_map_context(m, true, default_schedule);
+		int new_id = add_map_context(m, true, default_schedule, current_addon_);
 		switch_context(new_id);
 	} else {
-		replace_map_context(m, true, default_schedule);
+		replace_map_context(m, true, default_schedule, current_addon_);
 	}
 }
 
 void context_manager::new_scenario(int width, int height, const t_translation::terrain_code& fill, bool new_context)
 {
-	const config& default_schedule = game_config_.find_child("editor_times", "id", "empty");
+	auto default_schedule = game_config_.find_child("editor_times", "id", "empty");
 	editor_map m(width, height, fill);
 
 	if(new_context) {
-		int new_id = add_map_context(m, false, default_schedule);
+		int new_id = add_map_context(m, false, *default_schedule, current_addon_);
 		switch_context(new_id);
 	} else {
-		replace_map_context(m, false, default_schedule);
+		replace_map_context(m, false, *default_schedule, current_addon_);
 	}
 
 	// Give the new scenario an initial side.
@@ -1018,11 +1040,11 @@ void context_manager::create_default_context()
 		t_translation::terrain_code default_terrain =
 			t_translation::read_terrain_code(game_config::default_terrain);
 
-		const config& default_schedule = game_config_.find_child("editor_times", "id", "empty");
-		add_map_context(editor_map(44, 33, default_terrain), true, default_schedule);
+		const config& default_schedule = game_config_.find_mandatory_child("editor_times", "id", "empty");
+		add_map_context(editor_map(44, 33, default_terrain), true, default_schedule, current_addon_);
 	} else {
 		for(const std::string& filename : saved_windows_) {
-			add_map_context(game_config_, filename);
+			add_map_context(game_config_, filename, current_addon_);
 		}
 
 		saved_windows_.clear();
@@ -1049,7 +1071,7 @@ void context_manager::close_current_context()
 void context_manager::switch_context(const int index, const bool force)
 {
 	if(index < 0 || static_cast<std::size_t>(index) >= map_contexts_.size()) {
-		WRN_ED << "Invalid index in switch map context: " << index << std::endl;
+		WRN_ED << "Invalid index in switch map context: " << index;
 		return;
 	}
 
@@ -1079,7 +1101,7 @@ void context_manager::set_window_title()
 	}
 
 	const std::string& wm_title_string = name + " - " + game_config::get_default_title_string();
-	CVideo::get_singleton().set_window_title(wm_title_string);
+	video::set_window_title(wm_title_string);
 }
 
 } //Namespace editor
