@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -30,6 +30,17 @@
 #include "exceptions.hpp"
 #include "game_version.hpp"
 
+namespace game_config {
+extern std::string path;
+extern std::string default_preferences_path;
+extern bool check_migration;
+
+/** observer team name used for observer team chat */
+extern const std::string observer_team_name;
+
+extern int cache_compression_level;
+}
+
 class config;
 class game_config_view;
 struct SDL_RWops;
@@ -39,7 +50,12 @@ namespace filesystem {
 using scoped_istream = std::unique_ptr<std::istream>;
 using scoped_ostream = std::unique_ptr<std::ostream>;
 
-typedef std::unique_ptr<SDL_RWops, void(*)(SDL_RWops*)> rwops_ptr;
+struct sdl_rwops_deleter
+{
+	void operator()(SDL_RWops*) const noexcept;
+};
+
+using rwops_ptr = std::unique_ptr<SDL_RWops, sdl_rwops_deleter>;
 
 rwops_ptr make_read_RWops(const std::string &path);
 rwops_ptr make_write_RWops(const std::string &path);
@@ -88,39 +104,7 @@ private:
 	std::vector<std::string> directory_patterns_;
 };
 
-static const blacklist_pattern_list default_blacklist{
-	{
-		/* Blacklist dot-files/dirs, which are hidden files in UNIX platforms */
-		".+",
-		"#*#",
-		"*~",
-		"*-bak",
-		"*.swp",
-		"*.pbl",
-		"*.ign",
-		"_info.cfg",
-		"*.exe",
-		"*.bat",
-		"*.cmd",
-		"*.com",
-		"*.scr",
-		"*.sh",
-		"*.js",
-		"*.vbs",
-		"*.o",
-		"*.ini",
-		/* Remove junk created by certain file manager ;) */
-		"Thumbs.db",
-		/* Eclipse plugin */
-		"*.wesnoth",
-		"*.project",
-	},
-	{
-		".+",
-		/* macOS metadata-like cruft (http://floatingsun.net/2007/02/07/whats-with-__macosx-in-zip-files/) */
-		"__MACOSX",
-	}
-};
+extern const blacklist_pattern_list default_blacklist;
 
 /**
  * Get a list of all files and/or directories in a given directory.
@@ -149,9 +133,12 @@ std::string get_credentials_file();
 std::string get_default_prefs_file();
 std::string get_save_index_file();
 std::string get_saves_dir();
+std::string get_wml_persist_dir();
 std::string get_intl_dir();
 std::string get_screenshot_dir();
+std::string get_addons_data_dir();
 std::string get_addons_dir();
+std::string get_current_editor_dir(const std::string& addon_id);
 const std::string get_version_path_suffix(const version_info& version);
 const std::string& get_version_path_suffix();
 
@@ -162,10 +149,16 @@ const std::string& get_version_path_suffix();
 std::string get_next_filename(const std::string& name, const std::string& extension);
 void set_user_config_dir(const std::string& path);
 void set_user_data_dir(std::string path);
+void set_cache_dir(const std::string& path);
 
 std::string get_user_config_dir();
 std::string get_user_data_dir();
+std::string get_logs_dir();
 std::string get_cache_dir();
+std::string get_legacy_editor_dir();
+std::string get_core_images_dir();
+
+bool rename_dir(const std::string& old_dir, const std::string& new_dir);
 
 struct other_version_dir
 {
@@ -209,10 +202,13 @@ bool looks_like_pbl(const std::string& file);
 
 /** Basic disk I/O - read file. */
 std::string read_file(const std::string& fname);
+std::vector<uint8_t> read_file_binary(const std::string& fname);
+std::string read_file_as_data_uri(const std::string& fname);
+
 filesystem::scoped_istream istream_file(const std::string& fname, bool treat_failure_as_error = true);
 filesystem::scoped_ostream ostream_file(const std::string& fname, std::ios_base::openmode mode = std::ios_base::binary, bool create_directory = true);
 /** Throws io_exception if an error occurs. */
-void write_file(const std::string& fname, const std::string& data);
+void write_file(const std::string& fname, const std::string& data, std::ios_base::openmode mode = std::ios_base::binary);
 /**
  * Read a file and then writes it back out.
  *
@@ -222,6 +218,7 @@ void write_file(const std::string& fname, const std::string& data);
 void copy_file(const std::string& src, const std::string& dest);
 
 std::string read_map(const std::string& name);
+std::string read_scenario(const std::string& name);
 
 /**
  * Creates a directory if it does not exist already.
@@ -469,4 +466,10 @@ std::string get_program_invocation(const std::string &program_name);
  */
 std::string get_localized_path(const std::string& file, const std::string& suff = "");
 
+/**
+ * Returns the add-on ID from a path.
+ * aka the directory directly following the "add-ons" folder, or an empty string if none is found.
+ */
+std::string get_addon_id_from_path(const std::string& location);
+
 }
