@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -15,9 +15,12 @@
 
 #include "floating_label.hpp"
 
-#include "display.hpp"
+#include "draw.hpp"
+#include "draw_manager.hpp"
+#include "font/standard_colors.hpp"
 #include "font/text.hpp"
 #include "log.hpp"
+#include "sdl/utils.hpp"
 #include "video.hpp"
 
 #include <map>
@@ -30,6 +33,9 @@ static lg::log_domain log_font("font");
 #define WRN_FT LOG_STREAM(warn, log_font)
 #define ERR_FT LOG_STREAM(err, log_font)
 
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+
 namespace
 {
 typedef std::map<int, font::floating_label> label_map;
@@ -37,25 +43,21 @@ label_map labels;
 int label_id = 1;
 
 std::stack<std::set<int>> label_contexts;
+
 }
 
 namespace font
 {
 floating_label::floating_label(const std::string& text, const surface& surf)
-#if 0
-	: img_(),
-#else
-	: surf_(surf)
-	, buf_(nullptr)
-	, buf_pos_()
-#endif
+	: tex_()
+	, screen_loc_()
+	, alpha_(0)
 	, fadeout_(0)
 	, time_start_(0)
 	, text_(text)
 	, font_size_(SIZE_SMALL)
 	, color_(NORMAL_COLOR)
-	, bgcolor_()
-	, bgalpha_(0)
+	, bgcolor_(0, 0, 0, SDL_ALPHA_TRANSPARENT)
 	, xpos_(0)
 	, ypos_(0)
 	, xmove_(0)
@@ -63,13 +65,16 @@ floating_label::floating_label(const std::string& text, const surface& surf)
 	, lifetime_(-1)
 	, width_(-1)
 	, height_(-1)
-	, clip_rect_(CVideo::get_singleton().screen_area())
+	, clip_rect_(video::game_canvas())
 	, visible_(true)
 	, align_(CENTER_ALIGN)
 	, border_(0)
 	, scroll_(ANCHOR_LABEL_SCREEN)
 	, use_markup_(true)
 {
+	if (surf.get()) {
+		tex_ = texture(surf);
+	}
 }
 
 void floating_label::move(double xmove, double ymove)
@@ -90,110 +95,131 @@ int floating_label::xpos(std::size_t width) const
 	return xpos;
 }
 
-surface floating_label::create_surface()
+rect floating_label::get_bg_rect(const rect& text_rect) const
 {
-	if(!surf_) {
-		font::pango_text& text = font::get_text_renderer();
-
-		text.set_link_aware(false)
-			.set_family_class(font::FONT_SANS_SERIF)
-			.set_font_size(font_size_)
-			.set_font_style(font::pango_text::STYLE_NORMAL)
-			.set_alignment(PANGO_ALIGN_LEFT)
-			.set_foreground_color(color_)
-			.set_maximum_width(width_ < 0 ? clip_rect_.w : width_)
-			.set_maximum_height(height_ < 0 ? clip_rect_.h : height_, true)
-			.set_ellipse_mode(PANGO_ELLIPSIZE_END)
-			.set_characters_per_line(0);
-
-		// ignore last '\n'
-		if(!text_.empty() && *(text_.rbegin()) == '\n') {
-			text.set_text(std::string(text_.begin(), text_.end() - 1), use_markup_);
-		} else {
-			text.set_text(text_, use_markup_);
-		}
-
-		surface foreground = text.render();
-
-		if(foreground == nullptr) {
-			ERR_FT << "could not create floating label's text" << std::endl;
-			return nullptr;
-		}
-
-		// combine foreground text with its background
-		if(bgalpha_ != 0) {
-			// background is a dark tooltip box
-			surface background(foreground->w + border_ * 2, foreground->h + border_ * 2);
-
-			if(background == nullptr) {
-				ERR_FT << "could not create tooltip box" << std::endl;
-				return surf_ = foreground;
-			}
-
-			uint32_t color = SDL_MapRGBA(foreground->format, bgcolor_.r, bgcolor_.g, bgcolor_.b, bgalpha_);
-			sdl::fill_surface_rect(background, nullptr, color);
-
-			// we make the text less transparent, because the blitting on the
-			// dark background will darken the anti-aliased part.
-			// This 1.13 value seems to restore the brightness of version 1.4
-			// (where the text was blitted directly on screen)
-			adjust_surface_alpha(foreground, ftofxp(1.13));
-
-			SDL_Rect r{border_, border_, 0, 0};
-			adjust_surface_alpha(foreground, SDL_ALPHA_OPAQUE);
-			sdl_blit(foreground, nullptr, background, &r);
-
-			surf_ = background;
-		} else {
-			// background is blurred shadow of the text
-			surface background(foreground->w + 4, foreground->h + 4);
-			sdl::fill_surface_rect(background, nullptr, 0);
-			SDL_Rect r{2, 2, 0, 0};
-			sdl_blit(foreground, nullptr, background, &r);
-			background = shadow_image(background);
-
-			if(background == nullptr) {
-				ERR_FT << "could not create floating label's shadow" << std::endl;
-				return surf_ = foreground;
-			}
-			sdl_blit(foreground, nullptr, background, &r);
-			surf_ = background;
-		}
-	}
-
-	return surf_;
+	return {
+		text_rect.x -  border_,
+		text_rect.y -  border_,
+		text_rect.w + (border_ * 2),
+		text_rect.h + (border_ * 2)
+	};
 }
 
-void floating_label::draw(int time, surface screen)
+bool floating_label::create_texture()
+{
+	if(video::headless()) {
+		return false;
+	}
+
+	if(tex_ != nullptr) {
+		// Already have a texture
+		return true;
+	}
+
+	if(text_.empty()) {
+		// Empty labels are unfortunately still used sometimes
+		return false;
+	}
+
+	DBG_FT << "creating floating label texture";
+	font::pango_text& text = font::get_text_renderer();
+
+	text.set_link_aware(false)
+		.set_family_class(font::FONT_SANS_SERIF)
+		.set_font_size(font_size_)
+		.set_font_style(font::pango_text::STYLE_NORMAL)
+		.set_alignment(PANGO_ALIGN_LEFT)
+		.set_foreground_color(color_)
+		.set_maximum_width(width_ < 0 ? clip_rect_.w : width_)
+		.set_maximum_height(height_ < 0 ? clip_rect_.h : height_, true)
+		.set_ellipse_mode(PANGO_ELLIPSIZE_END)
+		.set_characters_per_line(0)
+		.set_add_outline(bgcolor_.a == 0);
+
+	// ignore last '\n'
+	if(!text_.empty() && *(text_.rbegin()) == '\n') {
+		text.set_text(std::string(text_.begin(), text_.end() - 1), use_markup_);
+	} else {
+		text.set_text(text_, use_markup_);
+	}
+
+	tex_ = text.render_and_get_texture();
+	if(!tex_) {
+		ERR_FT << "could not create floating label's text";
+		return false;
+	}
+
+	return true;
+}
+
+void floating_label::undraw()
+{
+	DBG_FT << "undrawing floating label from " << screen_loc_;
+	draw_manager::invalidate_region(get_bg_rect(screen_loc_));
+	screen_loc_ = {};
+}
+
+void floating_label::update(int time)
+{
+	if(video::headless() || text_.empty()) {
+		return;
+	}
+
+	if(!create_texture()) {
+		ERR_FT << "failed to create texture for floating label";
+		return;
+	}
+
+	point new_pos = get_pos(time);
+	rect draw_loc {new_pos.x, new_pos.y, tex_.w(), tex_.h()};
+
+	uint8_t new_alpha = get_alpha(time);
+
+	// Invalidate former draw loc
+	draw_manager::invalidate_region(get_bg_rect(screen_loc_));
+
+	// Invalidate new draw loc in preparation
+	draw_manager::invalidate_region(get_bg_rect(draw_loc));
+
+	DBG_FT << "updating floating label from " << screen_loc_ << " to " << draw_loc;
+
+	screen_loc_ = draw_loc;
+	alpha_ = new_alpha;
+}
+
+void floating_label::draw()
 {
 	if(!visible_) {
-		buf_ = nullptr;
+		screen_loc_ = {};
 		return;
 	}
 
-	if(screen == nullptr) {
+	if(screen_loc_.empty()) {
 		return;
 	}
 
-	create_surface();
-	if(surf_ == nullptr) {
+	if(!tex_) {
+		ERR_DP << "trying to draw floating label with no texture!";
 		return;
 	}
 
-	if(buf_ == nullptr) {
-		buf_ = surface(surf_->w, surf_->h);
-		if(buf_ == nullptr) {
-			return;
-		}
+	if(!screen_loc_.overlaps(draw::get_clip().intersect(clip_rect_))) {
+		return;
 	}
 
-	SDL_Point pos = get_loc(time);
-	buf_pos_ = sdl::create_rect(pos.x, pos.y, surf_->w, surf_->h);
-	const clip_rect_setter clip_setter(screen, &clip_rect_);
-	//important: make a copy of buf_pos_ because sdl_blit modifies dst_rect.
-	SDL_Rect rect = buf_pos_;
-	sdl_copy_portion(screen, &rect, buf_, nullptr);
-	sdl_blit(get_surface(time), nullptr, screen, &rect);
+	DBG_FT << "drawing floating label to " << screen_loc_;
+
+	// Clip if appropriate.
+	auto clipper = draw::reduce_clip(clip_rect_);
+
+	// Draw background, if appropriate
+	if(bgcolor_.a != 0) {
+		draw::fill(get_bg_rect(screen_loc_), bgcolor_);
+	}
+
+	// Apply the label texture to the screen.
+	tex_.set_alpha_mod(alpha_);
+	draw::blit(tex_, screen_loc_);
 }
 
 void floating_label::set_lifetime(int lifetime, int fadeout)
@@ -204,37 +230,30 @@ void floating_label::set_lifetime(int lifetime, int fadeout)
 }
 
 
-SDL_Point floating_label::get_loc(int time)
+point floating_label::get_pos(int time)
 {
 	int time_alive = get_time_alive(time);
 	return {
-		static_cast<int>(time_alive * xmove_ + xpos(surf_->w)),
+		static_cast<int>(time_alive * xmove_ + xpos(tex_.w())),
 		static_cast<int>(time_alive * ymove_ + ypos_)
- 	};
+	};
 }
 
-surface floating_label::get_surface(int time)
+uint8_t floating_label::get_alpha(int time)
 {
 	if(lifetime_ >= 0 && fadeout_ > 0) {
 		int time_alive = get_time_alive(time);
-		if(time_alive >= lifetime_ && surf_ != nullptr) {
+		if(time_alive >= lifetime_ && tex_ != nullptr) {
 			// fade out moving floating labels
-			int alpha_add = -255 * (time_alive - lifetime_) / fadeout_;
-			return adjust_surface_alpha_add(surf_, alpha_add);
+			int alpha_sub = 255 * (time_alive - lifetime_) / fadeout_;
+			if (alpha_sub >= 255) {
+				return 0;
+			} else {
+				return 255 - alpha_sub;
+			}
 		}
 	}
-	return surf_;
-}
-
-void floating_label::undraw(surface screen)
-{
-	if(screen == nullptr || buf_ == nullptr) {
-		return;
-	}
-
-	const clip_rect_setter clip_setter(screen, &clip_rect_);
-	SDL_Rect rect = buf_pos_;
-	sdl_blit(buf_, nullptr, screen, &rect);
+	return 255;
 }
 
 int add_floating_label(const floating_label& flabel)
@@ -273,7 +292,12 @@ void remove_floating_label(int handle, int fadeout)
 		if(fadeout > 0) {
 			i->second.set_lifetime(0, fadeout);
 			return;
+		} else if(fadeout < 0) {
+			i->second.set_lifetime(0, i->second.get_fade_time());
+			return;
 		}
+		// Queue a redraw of where the label was.
+		i->second.undraw();
 		labels.erase(i);
 	}
 
@@ -294,9 +318,9 @@ SDL_Rect get_floating_label_rect(int handle)
 {
 	const label_map::iterator i = labels.find(handle);
 	if(i != labels.end()) {
-		const surface surf = i->second.create_surface();
-		if(surf != nullptr) {
-			return {0, 0, surf->w, surf->h};
+		if (i->second.create_texture()) {
+			SDL_Point size = i->second.get_draw_size();
+			return {0, 0, size.x, size.y};
 		}
 	}
 	return sdl::empty_rect;
@@ -322,25 +346,24 @@ floating_label_context::~floating_label_context()
 	label_contexts.pop();
 }
 
-void draw_floating_labels(surface screen)
+void draw_floating_labels()
 {
 	if(label_contexts.empty()) {
 		return;
 	}
-	int time = SDL_GetTicks();
 
 	const std::set<int>& context = label_contexts.top();
 
 	// draw the labels in the order they were added, so later added labels (likely to be tooltips)
 	// are displayed over earlier added labels.
-	for(label_map::iterator i = labels.begin(); i != labels.end(); ++i) {
-		if(context.count(i->first) > 0) {
-			i->second.draw(time, screen);
+	for(auto& [id, label] : labels) {
+		if(context.count(id) > 0) {
+			label.draw();
 		}
 	}
 }
 
-void undraw_floating_labels(surface screen)
+void update_floating_labels()
 {
 	if(label_contexts.empty()) {
 		return;
@@ -349,17 +372,16 @@ void undraw_floating_labels(surface screen)
 
 	std::set<int>& context = label_contexts.top();
 
-	//undraw labels in reverse order, so that a LIFO process occurs, and the screen is restored
-	//into the exact state it started in.
-	for(label_map::reverse_iterator i = labels.rbegin(); i != labels.rend(); ++i) {
-		if(context.count(i->first) > 0) {
-			i->second.undraw(screen);
+	for(auto& [id, label] : labels) {
+		if(context.count(id) > 0) {
+			label.update(time);
 		}
 	}
 
 	//remove expired labels
 	for(label_map::iterator j = labels.begin(); j != labels.end(); ) {
 		if(context.count(j->first) > 0 && j->second.expired(time)) {
+			DBG_FT << "removing expired floating label " << j->first;
 			context.erase(j->first);
 			labels.erase(j++);
 		} else {
@@ -367,4 +389,5 @@ void undraw_floating_labels(surface screen)
 		}
 	}
 }
+
 }
