@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -30,24 +30,6 @@ static lg::log_domain log_engine("engine");
 
 namespace game_config
 {
-//
-// Path info
-//
-#ifdef WESNOTH_PATH
-std::string path = WESNOTH_PATH;
-#else
-std::string path = "";
-#endif
-
-#ifdef DEFAULT_PREFS_PATH
-std::string default_preferences_path = DEFAULT_PREFS_PATH;
-#else
-std::string default_preferences_path = "";
-#endif
-bool check_migration = false;
-
-std::string wesnoth_program_dir;
-
 //
 // Gameplay constants
 //
@@ -78,18 +60,13 @@ std::vector<unsigned int> zoom_levels {36, 72, 144};
 //
 double hp_bar_scaling  = 0.666;
 double xp_bar_scaling  = 0.5;
-double hex_brightening = 1.25;
 
 //
 // Misc
 //
-int cache_compression_level = 6;
-
 unsigned lobby_network_timer  = 100;
 unsigned lobby_refresh        = 4000;
 
-const std::string observer_team_name = "observer";
-
 const std::size_t max_loop = 65536;
 
 std::vector<server_info> server_list;
@@ -114,17 +91,20 @@ bool
 const bool& debug = debug_impl;
 
 void set_debug(bool new_debug) {
+    // TODO: remove severity static casts and fix issue #7894
 	if(debug_impl && !new_debug) {
 		// Turning debug mode off; decrease deprecation severity
-		int severity;
+		lg::severity severity;
 		if(lg::get_log_domain_severity("deprecation", severity)) {
-			lg::set_log_domain_severity("deprecation", severity - 2);
+            int severityInt = static_cast<int>(severity);
+			lg::set_log_domain_severity("deprecation", static_cast<lg::severity>(severityInt - 2));
 		}
 	} else if(!debug_impl && new_debug) {
 		// Turning debug mode on; increase deprecation severity
-		int severity;
+        lg::severity severity;
 		if(lg::get_log_domain_severity("deprecation", severity)) {
-			lg::set_log_domain_severity("deprecation", severity + 2);
+            int severityInt = static_cast<int>(severity);
+			lg::set_log_domain_severity("deprecation", static_cast<lg::severity>(severityInt + 2));
 		}
 	}
 	debug_impl = new_debug;
@@ -160,18 +140,17 @@ std::vector<color_t> red_green_scale_text;
 static std::vector<color_t> blue_white_scale;
 static std::vector<color_t> blue_white_scale_text;
 
-std::map<std::string, color_range> team_rgb_range;
+std::map<std::string, color_range, std::less<>> team_rgb_range;
 // Map [color_range]id to [color_range]name, or "" if no name
-std::map<std::string, t_string> team_rgb_name;
+std::map<std::string, t_string, std::less<>> team_rgb_name;
 
-std::map<std::string, std::vector<color_t>> team_rgb_colors;
+std::map<std::string, std::vector<color_t>, std::less<>> team_rgb_colors;
 
 std::vector<std::string> default_colors;
 
 namespace colors
 {
 std::string ally_orb_color;
-std::string disengaged_orb_color;
 std::string enemy_orb_color;
 std::string moved_orb_color;
 std::string partial_orb_color;
@@ -229,6 +208,7 @@ std::string
 	level,
 	ellipsis,
 	missing,
+	blank,
 	// notifications icon
 	app_icon = "images/icons/icon-game.png";
 
@@ -297,7 +277,12 @@ void load_config(const config &v)
 	if(!zoom_levels_str.empty()) {
 		zoom_levels.clear();
 		std::transform(zoom_levels_str.begin(), zoom_levels_str.end(), std::back_inserter(zoom_levels), [](const std::string zoom) {
-			return static_cast<int>(std::stold(zoom) * tile_size);
+			int z = std::stoi(zoom);
+			if((z / 4) * 4 != z) {
+				ERR_NG << "zoom level " << z << " is not divisible by 4."
+					<< " This will cause graphical glitches!";
+			}
+			return z;
 		});
 	}
 
@@ -307,7 +292,7 @@ void load_config(const config &v)
 	default_victory_music = utils::split(v["default_victory_music"].str());
 	default_defeat_music  = utils::split(v["default_defeat_music"].str());
 
-	if(const config& i = v.child("colors")){
+	if(auto i = v.optional_child("colors")){
 		using namespace game_config::colors;
 
 		moved_orb_color    = i["moved_orb_color"].str();
@@ -315,18 +300,17 @@ void load_config(const config &v)
 		partial_orb_color  = i["partial_orb_color"].str();
 		enemy_orb_color    = i["enemy_orb_color"].str();
 		ally_orb_color     = i["ally_orb_color"].str();
-		disengaged_orb_color = i["disengaged_orb_color"].str();
 	} // colors
 
 	show_ally_orb     = v["show_ally_orb"].to_bool(true);
 	show_enemy_orb    = v["show_enemy_orb"].to_bool(false);
 	show_moved_orb    = v["show_moved_orb"].to_bool(true);
-	show_partial_orb  = v["show_partly_orb"].to_bool(true);
+	show_partial_orb  = v["show_partial_orb"].to_bool(true);
 	show_status_on_ally_orb = v["show_status_on_ally_orb"].to_bool(true);
 	show_unmoved_orb  = v["show_unmoved_orb"].to_bool(true);
 	show_disengaged_orb = v["show_disengaged_orb"].to_bool(true);
 
-	if(const config& i = v.child("images")){
+	if(auto i = v.optional_child("images")){
 		using namespace game_config::images;
 
 		game_title            = i["game_title"].str();
@@ -363,11 +347,11 @@ void load_config(const config &v)
 		level      = i["level"].str();
 		ellipsis   = i["ellipsis"].str();
 		missing    = i["missing"].str();
+		blank      = i["blank"].str();
 	} // images
 
 	hp_bar_scaling  = v["hp_bar_scaling"].to_double(0.666);
 	xp_bar_scaling  = v["xp_bar_scaling"].to_double(0.5);
-	hex_brightening = v["hex_brightening"].to_double(1.25);
 
 	foot_speed_prefix   = utils::split(v["footprint_prefix"]);
 	foot_teleport_enter = v["footprint_teleport_enter"].str();
@@ -396,7 +380,7 @@ void load_config(const config &v)
 			try {
 				color_vec.push_back(color_t::from_hex_string(s));
 			} catch(const std::invalid_argument& e) {
-				ERR_NG << "Error parsing color list '" << key << "'.\n" << e.what() << std::endl;
+				ERR_NG << "Error parsing color list '" << key << "'.\n" << e.what();
 				color_vec.push_back(fallback);
 			}
 		}
@@ -412,13 +396,13 @@ void load_config(const config &v)
 	server_list.clear();
 
 	for(const config& server : v.child_range("server")) {
-        server_info sinf;
-        sinf.name = server["name"].str();
-        sinf.address = server["address"].str();
-        server_list.push_back(sinf);
+		server_info sinf;
+		sinf.name = server["name"].str();
+		sinf.address = server["address"].str();
+		server_list.push_back(sinf);
 	}
 
-	if(const config& s = v.child("sounds")) {
+	if(auto s = v.optional_child("sounds")) {
 		using namespace game_config::sounds;
 
 		const auto load_attribute = [](const config& c, const std::string& key, std::string& member) {
@@ -427,26 +411,26 @@ void load_config(const config &v)
 			}
 		};
 
-		load_attribute(s, "turn_bell",        turn_bell);
-		load_attribute(s, "timer_bell",       timer_bell);
-		load_attribute(s, "public_message",   public_message);
-		load_attribute(s, "private_message",  private_message);
-		load_attribute(s, "friend_message",   friend_message);
-		load_attribute(s, "server_message",   server_message);
-		load_attribute(s, "player_joins",     player_joins);
-		load_attribute(s, "player_leaves",    player_leaves);
-		load_attribute(s, "game_created",     game_created);
-		load_attribute(s, "game_user_arrive", game_user_arrive);
-		load_attribute(s, "game_user_leave",  game_user_leave);
-		load_attribute(s, "ready_for_start",  ready_for_start);
-		load_attribute(s, "game_has_begun",   game_has_begun);
+		load_attribute(*s, "turn_bell",        turn_bell);
+		load_attribute(*s, "timer_bell",       timer_bell);
+		load_attribute(*s, "public_message",   public_message);
+		load_attribute(*s, "private_message",  private_message);
+		load_attribute(*s, "friend_message",   friend_message);
+		load_attribute(*s, "server_message",   server_message);
+		load_attribute(*s, "player_joins",     player_joins);
+		load_attribute(*s, "player_leaves",    player_leaves);
+		load_attribute(*s, "game_created",     game_created);
+		load_attribute(*s, "game_user_arrive", game_user_arrive);
+		load_attribute(*s, "game_user_leave",  game_user_leave);
+		load_attribute(*s, "ready_for_start",  ready_for_start);
+		load_attribute(*s, "game_has_begun",   game_has_begun);
 
-		if(const config & ss = s.child("status")) {
+		if(auto ss = s->optional_child("status")) {
 			using namespace game_config::sounds::status;
 
-			load_attribute(ss, "poisoned",  poisoned);
-			load_attribute(ss, "slowed",    slowed);
-			load_attribute(ss, "petrified", petrified);
+			load_attribute(*ss, "poisoned",  poisoned);
+			load_attribute(*ss, "slowed",    slowed);
+			load_attribute(*ss, "petrified", petrified);
 		}
 	}
 }
@@ -479,7 +463,7 @@ void add_color_info(const game_config_view& v, bool build_defaults)
 		team_rgb_range.emplace(id, color_range(temp));
 		team_rgb_name.emplace(id, teamC["name"].t_str());
 
-		LOG_NG << "registered color range '" << id << "': " << team_rgb_range[id].debug() << '\n';
+		LOG_NG << "registered color range '" << id << "': " << team_rgb_range[id].debug();
 
 		// Ggenerate palette of same name;
 		std::vector<color_t> tp = palette(team_rgb_range[id]);
@@ -499,12 +483,12 @@ void add_color_info(const game_config_view& v, bool build_defaults)
 				try {
 					temp.push_back(color_t::from_hex_string(s));
 				} catch(const std::invalid_argument&) {
-					ERR_NG << "Invalid color in palette: " << s << std::endl;
+					ERR_NG << "Invalid color in palette: " << s;
 				}
 			}
 
 			team_rgb_colors.emplace(rgb.first, temp);
-			LOG_NG << "registered color palette: " << rgb.first << '\n';
+			LOG_NG << "registered color palette: " << rgb.first;
 		}
 	}
 }
@@ -517,7 +501,7 @@ void reset_color_info()
 	team_rgb_range.clear();
 }
 
-const color_range& color_info(const std::string& name)
+const color_range& color_info(std::string_view name)
 {
 	auto i = team_rgb_range.find(name);
 	if(i != team_rgb_range.end()) {
@@ -537,7 +521,7 @@ const color_range& color_info(const std::string& name)
 	return color_info(name);
 }
 
-const std::vector<color_t>& tc_info(const std::string& name)
+const std::vector<color_t>& tc_info(std::string_view name)
 {
 	auto i = team_rgb_colors.find(name);
 	if(i != team_rgb_colors.end()) {
@@ -550,7 +534,7 @@ const std::vector<color_t>& tc_info(const std::string& name)
 			temp.push_back(color_t::from_hex_string(s));
 		} catch(const std::invalid_argument&) {
 			static std::vector<color_t> stv;
-			ERR_NG << "Invalid color in palette: " << s << std::endl;
+			ERR_NG << "Invalid color in palette: " << s;
 			return stv;
 		}
 	}
@@ -559,22 +543,22 @@ const std::vector<color_t>& tc_info(const std::string& name)
 	return tc_info(name);
 }
 
-color_t red_to_green(int val, bool for_text)
+color_t red_to_green(double val, bool for_text)
 {
 	const std::vector<color_t>& color_scale = for_text ? red_green_scale_text : red_green_scale;
 
-	val = std::clamp(val, 0, 100);
-	const int lvl = (color_scale.size() - 1) * val / 100;
+	const double val_scaled = std::clamp(0.01 * val, 0.0, 1.0);
+	const int lvl = int(std::nearbyint((color_scale.size() - 1) * val_scaled));
 
 	return color_scale[lvl];
 }
 
-color_t blue_to_white(int val, bool for_text)
+color_t blue_to_white(double val, bool for_text)
 {
 	const std::vector<color_t>& color_scale = for_text ? blue_white_scale_text : blue_white_scale;
 
-	val = std::clamp(val, 0, 100);
-	const int lvl = (color_scale.size() - 1) * val / 100;
+	const double val_scaled = std::clamp(0.01 * val, 0.0, 1.0);
+	const int lvl = int(std::nearbyint((color_scale.size() - 1) * val_scaled));
 
 	return color_scale[lvl];
 }
