@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by Jörg Hinrichs <joerg.hinrichs@alice-dsl.de>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -81,7 +81,8 @@ bool game_load::execute(const game_config_view& cache_config, savegame::load_gam
 }
 
 game_load::game_load(const game_config_view& cache_config, savegame::load_game_metadata& data)
-	: filename_(data.filename)
+	: modal_dialog(window_id())
+	, filename_(data.filename)
 	, save_index_manager_(data.manager)
 	, change_difficulty_(register_bool("change_difficulty", true, data.select_difficulty))
 	, show_replay_(register_bool("show_replay", true, data.show_replay))
@@ -162,8 +163,8 @@ void game_load::populate_game_list()
 	games_ = save_index_manager_->get_saves_list();
 
 	for(const auto& game : games_) {
-		std::map<std::string, string_map> data;
-		string_map item;
+		widget_data data;
+		widget_item item;
 
 		std::string name = game.name();
 		utils::ellipsis_truncate(name, 40);
@@ -197,8 +198,8 @@ void game_load::display_savegame_internal(const savegame::save_info& game)
 	const std::string sprite_scale_mod = (formatter() << "~SCALE_INTO(" << game_config::tile_size << ',' << game_config::tile_size << ')').str();
 
 	for(const auto& leader : summary_.child_range("leader")) {
-		std::map<std::string, string_map> data;
-		string_map item;
+		widget_data data;
+		widget_item item;
 
 		// First, we evaluate whether the leader image as provided exists.
 		// If not, we try getting a binary path-independent path. If that still doesn't
@@ -230,12 +231,10 @@ void game_load::display_savegame_internal(const savegame::save_info& game)
 		item["label"] = leader["gold"];
 		data.emplace("leader_gold", item);
 
-		item["label"] = leader["units"];
+		// TRANSLATORS: "reserve" refers to units on the recall list
+		item["label"] = VGETTEXT("$active active, $reserve reserve", {{"active", leader["units"]}, {"reserve", leader["recall_units"]}});
 		data.emplace("leader_troops", item);
 
-		item["label"] = leader["recall_units"];
-		data.emplace("leader_reserves", item);
-
 		leader_list.add_row(data);
 	}
 
@@ -284,7 +283,7 @@ void game_load::display_savegame()
 		// Clear the UI widgets, show an error message.
 		const std::string preamble = _("The selected file is corrupt: ");
 		const std::string message = e.message.empty() ? "(no details)" : e.message;
-		ERR_GAMELOADDLG << preamble << message << "\n";
+		ERR_GAMELOADDLG << preamble << message;
 	}
 
 	if(!successfully_displayed_a_game) {
@@ -364,14 +363,15 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 
 	const std::string& campaign_type = cfg_summary["campaign_type"];
 	const std::string campaign_id = cfg_summary["campaign"];
+	auto campaign_type_enum = campaign_type::get_enum(campaign_type);
 
-	try {
-		switch(game_classification::CAMPAIGN_TYPE::string_to_enum(campaign_type).v) {
-			case game_classification::CAMPAIGN_TYPE::SCENARIO: {
+	if(campaign_type_enum) {
+		switch(*campaign_type_enum) {
+			case campaign_type::type::scenario: {
 				const config* campaign = nullptr;
 				if(!campaign_id.empty()) {
-					if(const config& c = cache_config_.find_child("campaign", "id", campaign_id)) {
-						campaign = &c;
+					if(auto c = cache_config_.find_child("campaign", "id", campaign_id)) {
+						campaign = c.ptr();
 					}
 				}
 
@@ -391,17 +391,17 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 				}
 				break;
 			}
-			case game_classification::CAMPAIGN_TYPE::MULTIPLAYER:
+			case campaign_type::type::multiplayer:
 				str << _("Multiplayer");
 				break;
-			case game_classification::CAMPAIGN_TYPE::TUTORIAL:
+			case campaign_type::type::tutorial:
 				str << _("Tutorial");
 				break;
-			case game_classification::CAMPAIGN_TYPE::TEST:
+			case campaign_type::type::test:
 				str << _("Test scenario");
 				break;
 		}
-	} catch(const bad_enum_cast&) {
+	} else {
 		str << campaign_type;
 	}
 
@@ -415,15 +415,14 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 		str << _("Scenario start");
 	}
 
-	str << "\n" << _("Difficulty: ");
-	try {
-		switch (game_classification::CAMPAIGN_TYPE::string_to_enum(campaign_type).v) {
-		case game_classification::CAMPAIGN_TYPE::SCENARIO:
-		case game_classification::CAMPAIGN_TYPE::MULTIPLAYER: {
+	if(campaign_type_enum) {
+		switch (*campaign_type_enum) {
+		case campaign_type::type::scenario:
+		case campaign_type::type::multiplayer: {
 			const config* campaign = nullptr;
 			if (!campaign_id.empty()) {
-				if (const config& c = cache_config_.find_child("campaign", "id", campaign_id)) {
-					campaign = &c;
+				if (auto c = cache_config_.find_child("campaign", "id", campaign_id)) {
+					campaign = c.ptr();
 				}
 			}
 
@@ -432,8 +431,9 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 			// For the latter do not show the difficulty - even though it will be listed as
 			// NORMAL -> Medium in the save file it should not be considered valid (GitHub Issue #5321)
 			if (campaign != nullptr) {
+				str << "\n" << _("Difficulty: ");
 				try {
-					const config& difficulty = campaign->find_child("difficulty", "define", cfg_summary["difficulty"]);
+					const config& difficulty = campaign->find_mandatory_child("difficulty", "define", cfg_summary["difficulty"]);
 					std::ostringstream ss;
 					ss << difficulty["label"] << " (" << difficulty["description"] << ")";
 					str << ss.str();
@@ -443,20 +443,14 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 					str << string_table[cfg_summary["difficulty"]];
 				}
 			}
-			else {
-				str << "—";
-			}
 
 			break;
 		}
-		case game_classification::CAMPAIGN_TYPE::TUTORIAL:
-		case game_classification::CAMPAIGN_TYPE::TEST:
-			str << "—";
+		case campaign_type::type::tutorial:
+		case campaign_type::type::test:
 			break;
 		}
-	}
-	catch (const bad_enum_cast&) {
-		str << "—";
+	} else {
 	}
 
 	if(!cfg_summary["version"].empty()) {
@@ -469,7 +463,7 @@ void game_load::evaluate_summary_string(std::stringstream& str, const config& cf
 		for(const auto& mod_id : active_mods) {
 			std::string mod_name;
 			try {
-				mod_name = cache_config_.find_child("modification", "id", mod_id)["name"].str();
+				mod_name = cache_config_.find_mandatory_child("modification", "id", mod_id)["name"].str();
 			} catch(const config::error&) {
 				// Fallback to nontranslatable mod id.
 				mod_name = "(" + mod_id + ")";
