@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2016 - 2022
+	Copyright (C) 2016 - 2024
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
 	This program is free software; you can redistribute it and/or modify
@@ -16,21 +16,21 @@
 
 #include "gui/dialogs/multiplayer/faction_select.hpp"
 
+#include "formatter.hpp"
 #include "game_config_manager.hpp"
+#include "gettext.hpp"
 #include "gui/auxiliary/find_widget.hpp"
 #include "gui/core/log.hpp"
-#include "gui/widgets/listbox.hpp"
-#include "gui/widgets/settings.hpp"
-#include "gui/widgets/image.hpp"
-#include "gui/widgets/label.hpp"
 #include "gui/widgets/button.hpp"
+#include "gui/widgets/drawing.hpp"
+#include "gui/widgets/label.hpp"
+#include "gui/widgets/listbox.hpp"
 #include "gui/widgets/menu_button.hpp"
+#include "gui/widgets/settings.hpp"
 #include "gui/widgets/toggle_button.hpp"
 #include "gui/widgets/window.hpp"
-#include "formatter.hpp"
-#include "gettext.hpp"
 #include "help/help.hpp"
-#include "preferences/game.hpp"	// for encountered_units
+#include "preferences/game.hpp" // for encountered_units
 #include "units/types.hpp"
 
 #include <functional>
@@ -41,7 +41,8 @@ namespace gui2::dialogs
 REGISTER_DIALOG(faction_select)
 
 faction_select::faction_select(ng::flg_manager& flg_manager, const std::string& color, const int side)
-	: flg_manager_(flg_manager)
+	: modal_dialog(window_id())
+	, flg_manager_(flg_manager)
 	, tc_color_(color)
 	, side_(side)
 	, last_faction_(flg_manager.current_faction_index())
@@ -68,7 +69,7 @@ void faction_select::pre_show(window& window)
 	gender_toggle_.set_member_states("random");
 
 	gender_toggle_.set_callback_on_value_change(
-		std::bind(&faction_select::on_gender_select, this));
+		std::bind(&faction_select::on_gender_select, this, std::placeholders::_2));
 
 	//
 	// Set up leader menu button
@@ -93,8 +94,8 @@ void faction_select::pre_show(window& window)
 	for(const config *s : flg_manager_.choosable_factions()) {
 		const config& side = *s;
 
-		std::map<std::string, string_map> data;
-		string_map item;
+		widget_data data;
+		widget_item item;
 
 		const std::string name = side["name"].str();
 		// flag_rgb here is unrelated to any handling in the unit class
@@ -198,9 +199,9 @@ void faction_select::profile_button_callback()
 	}
 }
 
-void faction_select::on_gender_select()
+void faction_select::on_gender_select(const std::string val)
 {
-	flg_manager_.set_current_gender(gender_toggle_.get_active_member_value());
+	flg_manager_.set_current_gender(val);
 
 	update_leader_image();
 }
@@ -211,10 +212,10 @@ void faction_select::update_leader_image()
 
 	if(const unit_type* ut = unit_types.find(flg_manager_.current_leader())) {
 		const unit_type& utg = ut->get_gender_unit_type(flg_manager_.current_gender());
-		leader_image = formatter() << utg.image() << "~RC(" << utg.flag_rgb() << ">" << tc_color_ << ")" << "~SCALE_INTO_SHARP(144,144)";
+		leader_image = formatter() << utg.image() << "~RC(" << utg.flag_rgb() << ">" << tc_color_ << ")";
 	}
 
-	find_widget<image>(get_window(), "leader_image", false).set_label(leader_image);
+	find_widget<drawing>(get_window(), "leader_image", false).set_label(leader_image);
 }
 
 void faction_select::post_show(window& /*window*/)
