@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009 - 2022
+	Copyright (C) 2009 - 2024
 	by Iris Morelle <shadowm2006@gmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -35,11 +35,11 @@ static lg::log_domain log_display("display");
 namespace image {
 
 /** Adds @a mod to the queue (unless mod is nullptr). */
-void modification_queue::push(modification * mod)
+void modification_queue::push(std::unique_ptr<modification> mod)
 {
 	// Null pointers do not get stored. (Shouldn't happen, but just in case.)
 	if(mod != nullptr) {
-		priorities_[mod->priority()].emplace_back(mod);
+		priorities_[mod->priority()].push_back(std::move(mod));
 	}
 }
 
@@ -78,7 +78,7 @@ modification * modification_queue::top() const
 namespace {
 
 /** A function used to parse modification arguments */
-using mod_parser = std::function<modification*(const std::string&)>;
+using mod_parser = std::function<std::unique_ptr<modification>(const std::string&)>;
 
 /** A map of all registered mod parsers
  *
@@ -94,12 +94,12 @@ std::map<std::string, mod_parser> mod_parsers;
  * @return A pointer to the decoded modification object
  * @retval nullptr if the string is invalid or a parser isn't found
  */
-modification* decode_modification(const std::string& encoded_mod)
+std::unique_ptr<modification> decode_modification(const std::string& encoded_mod)
 {
 	std::vector<std::string> split = utils::parenthetical_split(encoded_mod);
 
 	if(split.size() != 2) {
-		ERR_DP << "error parsing image modifications: " << encoded_mod << "\n";
+		ERR_DP << "error parsing image modifications: " << encoded_mod;
 		return nullptr;
 	}
 
@@ -107,7 +107,7 @@ modification* decode_modification(const std::string& encoded_mod)
 	std::string args = split[1];
 
 	if(mod_parsers.find(mod_type) == mod_parsers.end()) {
-		ERR_DP << "unknown image function in path: " << mod_type << '\n';
+		ERR_DP << "unknown image function in path: " << mod_type;
 		return nullptr;
 	}
 
@@ -120,11 +120,13 @@ modification* decode_modification(const std::string& encoded_mod)
 modification::imod_exception::imod_exception(const std::stringstream& message_stream)
 	: message(message_stream.str())
 {
+	this->store();
 }
 
 modification::imod_exception::imod_exception(const std::string& message)
 	: message(message)
 {
+	this->store();
 }
 
 /** Decodes the modification string
@@ -141,10 +143,8 @@ modification_queue modification::decode(const std::string& encoded_mods)
 	modification_queue mods;
 
 	for(const std::string& encoded_mod : utils::parenthetical_split(encoded_mods, '~')) {
-		modification* mod = decode_modification(encoded_mod);
-
-		if(mod) {
-			mods.push(mod);
+		if(auto mod = decode_modification(encoded_mod)) {
+			mods.push(std::move(mod));
 		}
 	}
 
@@ -197,6 +197,11 @@ surface gs_modification::operator()(const surface& src) const
 	return greyscale_image(src);
 }
 
+surface crop_transparency_modification::operator()(const surface& src) const
+{
+	return get_non_transparent_portion(src);
+}
+
 surface bw_modification::operator()(const surface& src) const
 {
 	return monochrome_image(src, threshold_);
@@ -289,7 +294,7 @@ surface adjust_alpha_modification::operator()(const surface & src) const
 	surface nsurf = src.clone();
 
 	if(nsurf == nullptr) {
-		std::cerr << "could not make neutral surface...\n";
+		PLAIN_LOG << "could not make neutral surface...";
 		return nullptr;
 	}
 
@@ -336,7 +341,7 @@ surface adjust_channels_modification::operator()(const surface & src) const
 	surface nsurf = src.clone();
 
 	if(nsurf == nullptr) {
-		std::cerr << "could not make neutral surface...\n";
+		PLAIN_LOG << "could not make neutral surface...";
 		return nullptr;
 	}
 
@@ -464,65 +469,41 @@ surface light_modification::operator()(const surface& src) const {
 
 surface scale_modification::operator()(const surface& src) const
 {
-	std::pair<int,int> sz = calculate_size(src);
+	point size = target_size_;
 
-	if(nn_) {
-		return scale_surface_sharp(src, sz.first, sz.second);
+	if(size.x <= 0) {
+		if(size.x < 0) {
+			ERR_DP << "width of " << fn_ << " is negative - resetting to original width";
+		}
+		size.x = src->w;
+	}
+
+	if(size.y <= 0) {
+		if(size.y < 0) {
+			ERR_DP << "height of " << fn_ << " is negative - resetting to original height";
+		}
+		size.y = src->h;
+	}
+
+	if(flags_ & PRESERVE_ASPECT_RATIO) {
+		const auto ratio = std::min(
+			static_cast<long double>(size.x) / src->w,
+			static_cast<long double>(size.y) / src->h
+		);
+
+		size = {
+			static_cast<int>(src->w * ratio),
+			static_cast<int>(src->h * ratio)
+		};
+	}
+
+	if(flags_ & SCALE_SHARP) {
+		return scale_surface_sharp(src, size.x, size.y);
 	} else {
-		return scale_surface_legacy(src, sz.first, sz.second);
+		return scale_surface_legacy(src, size.x, size.y);
 	}
 }
 
-std::pair<int,int> scale_exact_modification::calculate_size(const surface& src) const
-{
-	const int old_w = src->w;
-	const int old_h = src->h;
-	int w = get_w();
-	int h = get_h();
-
-	if(w <= 0) {
-		if(w < 0) {
-			ERR_DP << "width of " << fn_ << " is negative - resetting to original width" << std::endl;
-		}
-		w = old_w;
-	}
-
-	if(h <= 0) {
-		if(h < 0) {
-			ERR_DP << "height of " << fn_ << " is negative - resetting to original height" << std::endl;
-		}
-		h = old_h;
-	}
-
-	return {w, h};
-}
-
-std::pair<int,int> scale_into_modification::calculate_size(const surface& src) const
-{
-	const int old_w = src->w;
-	const int old_h = src->h;
-	long double w = get_w();
-	long double h = get_h();
-
-	if(w <= 0) {
-		if(w < 0) {
-			ERR_DP << "width of SCALE_INTO is negative - resetting to original width" << std::endl;
-		}
-		w = old_w;
-	}
-
-	if(h <= 0) {
-		if(h < 0) {
-			ERR_DP << "height of SCALE_INTO is negative - resetting to original height" << std::endl;
-		}
-		h = old_h;
-	}
-
-	long double ratio = std::min(w / old_w, h / old_h);
-
-	return {static_cast<int>(old_w * ratio), static_cast<int>(old_h * ratio)};
-}
-
 surface xbrz_modification::operator()(const surface& src) const
 {
 	if(z_ == 1) {
@@ -541,11 +522,11 @@ surface o_modification::operator()(const surface& src) const
 	surface nsurf = src.clone();
 
 	if(nsurf == nullptr) {
-		std::cerr << "could not make neutral surface...\n";
+		PLAIN_LOG << "could not make neutral surface...";
 		return nullptr;
 	}
 
-	uint16_t amount = ftofxp(opacity_);
+	uint8_t alpha_mod = float_to_color(opacity_);
 
 	{
 		surface_lock lock(nsurf);
@@ -561,7 +542,7 @@ surface o_modification::operator()(const surface& src) const
 				g = (*beg) >> 8;
 				b = (*beg);
 
-				alpha = std::min<unsigned>(static_cast<unsigned>(fxpmult(alpha,amount)), 255);
+				alpha = color_multiply(alpha, alpha_mod);
 				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
 			}
 
@@ -624,9 +605,9 @@ struct parse_mod_registration
  * @param args_var The name for the string argument provided
  */
 #define REGISTER_MOD_PARSER(type, args_var)                                                           \
-    static modification* parse_##type##_mod(const std::string&);                                      \
+    static std::unique_ptr<modification> parse_##type##_mod(const std::string&);                      \
     static parse_mod_registration parse_##type##_mod_registration_aux(#type, &parse_##type##_mod);    \
-    static modification* parse_##type##_mod(const std::string& args_var)                              \
+    static std::unique_ptr<modification> parse_##type##_mod(const std::string& args_var)              \
 
 // Color-range-based recoloring
 REGISTER_MOD_PARSER(TC, args)
@@ -634,14 +615,14 @@ REGISTER_MOD_PARSER(TC, args)
 	std::vector<std::string> params = utils::split(args,',');
 
 	if(params.size() < 2) {
-		ERR_DP << "too few arguments passed to the ~TC() function" << std::endl;
+		ERR_DP << "too few arguments passed to the ~TC() function";
 
 		return nullptr;
 	}
 
 	const int side_n = lexical_cast_default<int>(params[0], -1);
 	if(side_n < 1) {
-		ERR_DP << "Invalid side (" << side_n << ") passed to the ~TC() function\n";
+		ERR_DP << "Invalid side (" << side_n << ") passed to the ~TC() function";
 		return nullptr;
 	}
 
@@ -649,9 +630,8 @@ REGISTER_MOD_PARSER(TC, args)
 	// Pass argseters for RC functor
 	//
 	if(!game_config::tc_info(params[1]).size()){
-		ERR_DP << "could not load TC info for '" << params[1]
-		       << "' palette\n"
-		       << "bailing out from TC\n";
+		ERR_DP << "could not load TC info for '" << params[1] << "' palette";
+		ERR_DP << "bailing out from TC";
 
 		return nullptr;
 	}
@@ -663,15 +643,13 @@ REGISTER_MOD_PARSER(TC, args)
 
 		rc_map = recolor_range(new_color,old_color);
 	} catch(const config::error& e) {
-		ERR_DP << "caught config::error while processing TC: "
-		       << e.message
-		       << '\n'
-		       << "bailing out from TC\n";
+		ERR_DP << "caught config::error while processing TC: " << e.message;
+		ERR_DP << "bailing out from TC";
 
 		return nullptr;
 	}
 
-	return new rc_modification(rc_map);
+	return std::make_unique<rc_modification>(rc_map);
 }
 
 // Team-color-based color range selection and recoloring
@@ -695,14 +673,12 @@ REGISTER_MOD_PARSER(RC, args)
 	} catch (const config::error& e) {
 		ERR_DP
 			<< "caught config::error while processing color-range RC: "
-			<< e.message
-			<< '\n';
-		ERR_DP
-			<< "bailing out from RC\n";
+			<< e.message;
+		ERR_DP << "bailing out from RC";
 		rc_map.clear();
 	}
 
-	return new rc_modification(rc_map);
+	return std::make_unique<rc_modification>(rc_map);
 }
 
 // Palette switch
@@ -711,7 +687,7 @@ REGISTER_MOD_PARSER(PAL, args)
 	const std::vector<std::string> remap_params = utils::split(args,'>');
 
 	if(remap_params.size() < 2) {
-		ERR_DP << "not enough arguments passed to the ~PAL() function: " << args << "\n";
+		ERR_DP << "not enough arguments passed to the ~PAL() function: " << args;
 
 		return nullptr;
 	}
@@ -725,14 +701,13 @@ REGISTER_MOD_PARSER(PAL, args)
 			rc_map[old_palette[i]] = new_palette[i];
 		}
 
-		return new rc_modification(rc_map);
+		return std::make_unique<rc_modification>(rc_map);
 	} catch(const config::error& e) {
 		ERR_DP
 			<< "caught config::error while processing PAL function: "
-			<< e.message
-			<< '\n';
+			<< e.message;
 		ERR_DP
-			<< "bailing out from PAL\n";
+			<< "bailing out from PAL";
 
 		return nullptr;
 	}
@@ -744,7 +719,7 @@ REGISTER_MOD_PARSER(FL, args)
 	bool horiz = (args.empty() || args.find("horiz") != std::string::npos);
 	bool vert = (args.find("vert") != std::string::npos);
 
-	return new fl_modification(horiz, vert);
+	return std::make_unique<fl_modification>(horiz, vert);
 }
 
 // Rotations
@@ -755,19 +730,19 @@ REGISTER_MOD_PARSER(ROTATE, args)
 
 	switch(s) {
 		case 0:
-			return new rotate_modification();
+			return std::make_unique<rotate_modification>();
 			break;
 		case 1:
-			return new rotate_modification(
+			return std::make_unique<rotate_modification>(
 				lexical_cast_default<int>(slice_params[0]));
 			break;
 		case 2:
-			return new rotate_modification(
+			return std::make_unique<rotate_modification>(
 				lexical_cast_default<int>(slice_params[0]),
 				lexical_cast_default<int>(slice_params[1]));
 			break;
 		case 3:
-			return new rotate_modification(
+			return std::make_unique<rotate_modification>(
 				lexical_cast_default<int>(slice_params[0]),
 				lexical_cast_default<int>(slice_params[1]),
 				lexical_cast_default<int>(slice_params[2]));
@@ -779,7 +754,13 @@ REGISTER_MOD_PARSER(ROTATE, args)
 // Grayscale
 REGISTER_MOD_PARSER(GS, )
 {
-	return new gs_modification;
+	return std::make_unique<gs_modification>();
+}
+
+// crop transparent padding
+REGISTER_MOD_PARSER(CROP_TRANSPARENCY, )
+{
+	return std::make_unique<crop_transparency_modification>();
 }
 
 // Black and white
@@ -788,20 +769,20 @@ REGISTER_MOD_PARSER(BW, args)
 	const std::vector<std::string>& params = utils::split(args, ',');
 
 	if(params.size() != 1) {
-		ERR_DP << "~BW() requires  exactly one argument" << std::endl;
+		ERR_DP << "~BW() requires  exactly one argument";
 		return nullptr;
 	}
 
 	try {
 		int threshold = std::stoi(params[0]);
 		if(threshold < 0 || threshold > 255) {
-			ERR_DP << "~BW() argument out of range 0 - 255" << std::endl;
+			ERR_DP << "~BW() argument out of range 0 - 255";
 			return nullptr;
 		}  else {
-			return new bw_modification(threshold);
+			return std::make_unique<bw_modification>(threshold);
 		}
 	} catch (const std::invalid_argument&) {
-		ERR_DP << "unsupported argument in ~BW() function" << std::endl;
+		ERR_DP << "unsupported argument in ~BW() function";
 		return nullptr;
 	}
 }
@@ -809,7 +790,7 @@ REGISTER_MOD_PARSER(BW, args)
 // Sepia
 REGISTER_MOD_PARSER(SEPIA, )
 {
-	return new sepia_modification;
+	return std::make_unique<sepia_modification>();
 }
 
 // Negative
@@ -822,19 +803,19 @@ REGISTER_MOD_PARSER(NEG, args)
 			// apparently -1 may be a magic number
 			// but this is the threshold value required
 			// to fully invert a channel
-			return new negative_modification(-1,-1,-1);
+			return std::make_unique<negative_modification>(-1, -1, -1);
 			break;
 		case 1:
 			try {
 				int threshold = std::stoi(params[0]);
 				if(threshold < -1 || threshold > 255) {
-					ERR_DP << "unsupported argument value in ~NEG() function" << std::endl;
+					ERR_DP << "unsupported argument value in ~NEG() function";
 					return nullptr;
 				} else {
-					return new negative_modification(threshold, threshold, threshold);
+					return std::make_unique<negative_modification>(threshold, threshold, threshold);
 				}
 			} catch (const std::invalid_argument&) {
-				ERR_DP << "unsupported argument value in ~NEG() function" << std::endl;
+				ERR_DP << "unsupported argument value in ~NEG() function";
 				return nullptr;
 			}
 			break;
@@ -844,18 +825,18 @@ REGISTER_MOD_PARSER(NEG, args)
 				int thresholdGreen = std::stoi(params[1]);
 				int thresholdBlue = std::stoi(params[2]);
 				if(thresholdRed < -1 || thresholdRed > 255 || thresholdGreen < -1 || thresholdGreen > 255 || thresholdBlue < -1 || thresholdBlue > 255) {
-					ERR_DP << "unsupported argument value in ~NEG() function" << std::endl;
+					ERR_DP << "unsupported argument value in ~NEG() function";
 					return nullptr;
 				} else {
-					return new negative_modification(thresholdRed, thresholdGreen, thresholdBlue);
+					return std::make_unique<negative_modification>(thresholdRed, thresholdGreen, thresholdBlue);
 				}
 			} catch (const std::invalid_argument&) {
-				ERR_DP << "unsupported argument value in ~NEG() function" << std::endl;
+				ERR_DP << "unsupported argument value in ~NEG() function";
 				return nullptr;
 			}
 			break;
 		default:
-			ERR_DP << "~NEG() requires 0, 1 or 3 arguments" << std::endl;
+			ERR_DP << "~NEG() requires 0, 1 or 3 arguments";
 			return nullptr;
 	}
 
@@ -865,13 +846,13 @@ REGISTER_MOD_PARSER(NEG, args)
 // Plot Alpha
 REGISTER_MOD_PARSER(PLOT_ALPHA, )
 {
-	return new plot_alpha_modification;
+	return std::make_unique<plot_alpha_modification>();
 }
 
 // Wipe Alpha
 REGISTER_MOD_PARSER(WIPE_ALPHA, )
 {
-	return new wipe_alpha_modification;
+	return std::make_unique<wipe_alpha_modification>();
 }
 
 // Adjust Alpha
@@ -882,11 +863,11 @@ REGISTER_MOD_PARSER(ADJUST_ALPHA, args)
 	const std::vector<std::string>& params = utils::parenthetical_split(args, ',', "([", ")]");
 
 	if(params.size() != 1) {
-		ERR_DP << "~ADJUST_ALPHA() requires exactly 1 arguments" << std::endl;
+		ERR_DP << "~ADJUST_ALPHA() requires exactly 1 arguments";
 		return nullptr;
 	}
 
-	return new adjust_alpha_modification(params.at(0));
+	return std::make_unique<adjust_alpha_modification>(params.at(0));
 }
 
 // Adjust Channels
@@ -897,11 +878,11 @@ REGISTER_MOD_PARSER(CHAN, args)
 	const std::vector<std::string>& params = utils::parenthetical_split(args, ',', "([", ")]");
 
 	if(params.size() < 1 || params.size() > 4) {
-		ERR_DP << "~CHAN() requires 1 to 4 arguments" << std::endl;
+		ERR_DP << "~CHAN() requires 1 to 4 arguments";
 		return nullptr;
 	}
 
-	return new adjust_channels_modification(params);
+	return std::make_unique<adjust_channels_modification>(params);
 }
 
 // Color-shift
@@ -911,7 +892,7 @@ REGISTER_MOD_PARSER(CS, args)
 	const std::size_t s = factors.size();
 
 	if(s == 0) {
-		ERR_DP << "no arguments passed to the ~CS() function" << std::endl;
+		ERR_DP << "no arguments passed to the ~CS() function";
 		return nullptr;
 	}
 
@@ -926,7 +907,7 @@ REGISTER_MOD_PARSER(CS, args)
 		b = lexical_cast_default<int>(factors[2]);
 	}
 
-	return new cs_modification(r, g, b);
+	return std::make_unique<cs_modification>(r, g , b);
 }
 
 // Color blending
@@ -935,7 +916,7 @@ REGISTER_MOD_PARSER(BLEND, args)
 	const std::vector<std::string>& params = utils::split(args, ',');
 
 	if(params.size() != 4) {
-		ERR_DP << "~BLEND() requires exactly 4 arguments" << std::endl;
+		ERR_DP << "~BLEND() requires exactly 4 arguments";
 		return nullptr;
 	}
 
@@ -952,7 +933,7 @@ REGISTER_MOD_PARSER(BLEND, args)
 		opacity /= 100.0f;
 	}
 
-	return new blend_modification(
+	return std::make_unique<blend_modification>(
 		lexical_cast_default<int>(params[0]),
 		lexical_cast_default<int>(params[1]),
 		lexical_cast_default<int>(params[2]),
@@ -966,7 +947,7 @@ REGISTER_MOD_PARSER(CROP, args)
 	const std::size_t s = slice_params.size();
 
 	if(s == 0 || (s == 1 && slice_params[0].empty())) {
-		ERR_DP << "no arguments passed to the ~CROP() function" << std::endl;
+		ERR_DP << "no arguments passed to the ~CROP() function";
 		return nullptr;
 	}
 
@@ -984,7 +965,7 @@ REGISTER_MOD_PARSER(CROP, args)
 		slice_rect.h = lexical_cast_default<uint16_t, const std::string&>(slice_params[3]);
 	}
 
-	return new crop_modification(slice_rect);
+	return std::make_unique<crop_modification>(slice_rect);
 }
 
 static bool check_image(const image::locator& img, std::stringstream & message)
@@ -1002,12 +983,12 @@ REGISTER_MOD_PARSER(BLIT, args)
 	const std::size_t s = param.size();
 
 	if(s == 0 || (s == 1 && param[0].empty())){
-		ERR_DP << "no arguments passed to the ~BLIT() function" << std::endl;
+		ERR_DP << "no arguments passed to the ~BLIT() function";
 		return nullptr;
 	}
 
 	if(s > 3){
-		ERR_DP << "too many arguments passed to the ~BLIT() function" << std::endl;
+		ERR_DP << "too many arguments passed to the ~BLIT() function";
 		return nullptr;
 	}
 
@@ -1023,9 +1004,9 @@ REGISTER_MOD_PARSER(BLIT, args)
 	message << "~BLIT():";
 	if(!check_image(img, message))
 		return nullptr;
-	surface surf = get_image(img);
+	surface surf = get_surface(img);
 
-	return new blit_modification(surf, x, y);
+	return std::make_unique<blit_modification>(surf, x, y);
 }
 
 // Mask
@@ -1035,7 +1016,7 @@ REGISTER_MOD_PARSER(MASK, args)
 	const std::size_t s = param.size();
 
 	if(s == 0 || (s == 1 && param[0].empty())){
-		ERR_DP << "no arguments passed to the ~MASK() function" << std::endl;
+		ERR_DP << "no arguments passed to the ~MASK() function";
 		return nullptr;
 	}
 
@@ -1047,7 +1028,7 @@ REGISTER_MOD_PARSER(MASK, args)
 	}
 
 	if(x < 0 || y < 0) {
-		ERR_DP << "negative position arguments in ~MASK() function" << std::endl;
+		ERR_DP << "negative position arguments in ~MASK() function";
 		return nullptr;
 	}
 
@@ -1056,107 +1037,91 @@ REGISTER_MOD_PARSER(MASK, args)
 	message << "~MASK():";
 	if(!check_image(img, message))
 		return nullptr;
-	surface surf = get_image(img);
+	surface surf = get_surface(img);
 
-	return new mask_modification(surf, x, y);
+	return std::make_unique<mask_modification>(surf, x, y);
 }
 
 // Light
 REGISTER_MOD_PARSER(L, args)
 {
 	if(args.empty()){
-		ERR_DP << "no arguments passed to the ~L() function" << std::endl;
+		ERR_DP << "no arguments passed to the ~L() function";
 		return nullptr;
 	}
 
-	surface surf = get_image(args);
+	surface surf = get_surface(args);
 
-	return new light_modification(surf);
+	return std::make_unique<light_modification>(surf);
 }
 
+namespace
+{
+/** Common helper function to parse scaling IPF inputs. */
+std::optional<point> parse_scale_args(const std::string& args)
+{
+	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
+	const std::size_t s = scale_params.size();
+
+	if(s == 0 || (s == 1 && scale_params[0].empty())) {
+		return std::nullopt;
+	}
+
+	point size{0, 0};
+	size.x = lexical_cast_default<int, const std::string&>(scale_params[0]);
+
+	if(s > 1) {
+		size.y = lexical_cast_default<int, const std::string&>(scale_params[1]);
+	}
+
+	return size;
+}
+
+} // namespace
+
 // Scale
 REGISTER_MOD_PARSER(SCALE, args)
 {
-	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const std::size_t s = scale_params.size();
-
-	if(s == 0 || (s == 1 && scale_params[0].empty())) {
-		ERR_DP << "no arguments passed to the ~SCALE() function" << std::endl;
+	if(auto size = parse_scale_args(args)) {
+		constexpr uint8_t mode = scale_modification::SCALE_LINEAR | scale_modification::FIT_TO_SIZE;
+		return std::make_unique<scale_modification>(*size, "SCALE", mode);
+	} else {
+		ERR_DP << "no arguments passed to the ~SCALE() function";
 		return nullptr;
 	}
-
-	int w = 0, h = 0;
-
-	w = lexical_cast_default<int, const std::string&>(scale_params[0]);
-
-	if(s > 1) {
-		h = lexical_cast_default<int, const std::string&>(scale_params[1]);
-	}
-
-	return new scale_exact_modification(w, h, "SCALE", false);
 }
 
 REGISTER_MOD_PARSER(SCALE_SHARP, args)
 {
-	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const std::size_t s = scale_params.size();
-
-	if(s == 0 || (s == 1 && scale_params[0].empty())) {
-		ERR_DP << "no arguments passed to the ~SCALE_SHARP() function" << std::endl;
+	if(auto size = parse_scale_args(args)) {
+		constexpr uint8_t mode = scale_modification::SCALE_SHARP | scale_modification::FIT_TO_SIZE;
+		return std::make_unique<scale_modification>(*size, "SCALE_SHARP", mode);
+	} else {
+		ERR_DP << "no arguments passed to the ~SCALE_SHARP() function";
 		return nullptr;
 	}
-
-	int w = 0, h = 0;
-
-	w = lexical_cast_default<int, const std::string&>(scale_params[0]);
-
-	if(s > 1) {
-		h = lexical_cast_default<int, const std::string&>(scale_params[1]);
-	}
-
-	return new scale_exact_modification(w, h, "SCALE_SHARP", true);
 }
 
 REGISTER_MOD_PARSER(SCALE_INTO, args)
 {
-	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const std::size_t s = scale_params.size();
-
-	if(s == 0 || (s == 1 && scale_params[0].empty())) {
-		ERR_DP << "no arguments passed to the ~SCALE_INTO() function" << std::endl;
+	if(auto size = parse_scale_args(args)) {
+		constexpr uint8_t mode = scale_modification::SCALE_LINEAR | scale_modification::PRESERVE_ASPECT_RATIO;
+		return std::make_unique<scale_modification>(*size, "SCALE_INTO", mode);
+	} else {
+		ERR_DP << "no arguments passed to the ~SCALE_INTO() function";
 		return nullptr;
 	}
-
-	int w = 0, h = 0;
-
-	w = lexical_cast_default<int, const std::string&>(scale_params[0]);
-
-	if(s > 1) {
-		h = lexical_cast_default<int, const std::string&>(scale_params[1]);
-	}
-
-	return new scale_into_modification(w, h, "SCALE_INTO", false);
 }
 
 REGISTER_MOD_PARSER(SCALE_INTO_SHARP, args)
 {
-	const std::vector<std::string>& scale_params = utils::split(args, ',', utils::STRIP_SPACES);
-	const std::size_t s = scale_params.size();
-
-	if(s == 0 || (s == 1 && scale_params[0].empty())) {
-		ERR_DP << "no arguments passed to the ~SCALE_INTO_SHARP() function" << std::endl;
+	if(auto size = parse_scale_args(args)) {
+		constexpr uint8_t mode = scale_modification::SCALE_SHARP | scale_modification::PRESERVE_ASPECT_RATIO;
+		return std::make_unique<scale_modification>(*size, "SCALE_INTO_SHARP", mode);
+	} else {
+		ERR_DP << "no arguments passed to the ~SCALE_INTO_SHARP() function";
 		return nullptr;
 	}
-
-	int w = 0, h = 0;
-
-	w = lexical_cast_default<int, const std::string&>(scale_params[0]);
-
-	if(s > 1) {
-		h = lexical_cast_default<int, const std::string&>(scale_params[1]);
-	}
-
-	return new scale_into_modification(w, h, "SCALE_INTO_SHARP", true);
 }
 
 // xBRZ
@@ -1167,7 +1132,7 @@ REGISTER_MOD_PARSER(XBRZ, args)
 		z = 5; //only values 2 - 5 are permitted for xbrz scaling factors.
 	}
 
-	return new xbrz_modification(z);
+	return std::make_unique<xbrz_modification>(z);
 }
 
 // scale
@@ -1176,8 +1141,7 @@ REGISTER_MOD_PARSER(XBRZ, args)
 REGISTER_MOD_PARSER(BL, args)
 {
 	const int depth = std::max<int>(0, lexical_cast_default<int>(args));
-
-	return new bl_modification(depth);
+	return std::make_unique<bl_modification>(depth);
 }
 
 // Opacity-shift
@@ -1194,7 +1158,7 @@ REGISTER_MOD_PARSER(O, args)
 		num /= 100.0f;
 	}
 
-	return new o_modification(num);
+	return std::make_unique<o_modification>(num);
 }
 
 //
@@ -1205,24 +1169,21 @@ REGISTER_MOD_PARSER(O, args)
 REGISTER_MOD_PARSER(R, args)
 {
 	const int r = lexical_cast_default<int>(args);
-
-	return new cs_modification(r,0,0);
+	return std::make_unique<cs_modification>(r,0,0);
 }
 
 // Green component color-shift
 REGISTER_MOD_PARSER(G, args)
 {
 	const int g = lexical_cast_default<int>(args);
-
-	return new cs_modification(0,g,0);
+	return std::make_unique<cs_modification>(0,g,0);
 }
 
 // Blue component color-shift
 REGISTER_MOD_PARSER(B, args)
 {
 	const int b = lexical_cast_default<int>(args);
-
-	return new cs_modification(0,0,b);
+	return std::make_unique<cs_modification>(0,0,b);
 }
 
 REGISTER_MOD_PARSER(NOP, )
@@ -1253,7 +1214,7 @@ REGISTER_MOD_PARSER(BG, args)
 		c[i] = lexical_cast_default<int>(factors[i]);
 	}
 
-	return new background_modification(color_t(c[0], c[1], c[2], c[3]));
+	return std::make_unique<background_modification>(color_t(c[0], c[1], c[2], c[3]));
 }
 
 // Channel swap
@@ -1263,7 +1224,7 @@ REGISTER_MOD_PARSER(SWAP, args)
 
 	// accept 3 arguments (rgb) or 4 (rgba)
 	if(params.size() != 3 && params.size() != 4) {
-		ERR_DP << "incorrect number of arguments in ~SWAP() function, they must be 3 or 4" << std::endl;
+		ERR_DP << "incorrect number of arguments in ~SWAP() function, they must be 3 or 4";
 		return nullptr;
 	}
 
@@ -1278,7 +1239,7 @@ REGISTER_MOD_PARSER(SWAP, args)
 	} else if(params[0] == "alpha") {
 		redValue = ALPHA;
 	} else {
-		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0] << std::endl;
+		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0];
 		return nullptr;
 	}
 
@@ -1292,7 +1253,7 @@ REGISTER_MOD_PARSER(SWAP, args)
 	} else if(params[1] == "alpha") {
 		greenValue = ALPHA;
 	} else {
-		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0] << std::endl;
+		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0];
 		return nullptr;
 	}
 
@@ -1305,7 +1266,7 @@ REGISTER_MOD_PARSER(SWAP, args)
 	} else if(params[2] == "alpha") {
 		blueValue = ALPHA;
 	} else {
-		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0] << std::endl;
+		ERR_DP << "unsupported argument value in ~SWAP() function: " << params[0];
 		return nullptr;
 	}
 
@@ -1323,12 +1284,12 @@ REGISTER_MOD_PARSER(SWAP, args)
 		} else if(params[3] == "alpha") {
 			alphaValue = ALPHA;
 		} else {
-			ERR_DP << "unsupported argument value in ~SWAP() function: " << params[3] << std::endl;
+			ERR_DP << "unsupported argument value in ~SWAP() function: " << params[3];
 			return nullptr;
 		}
 	}
 
-	return new swap_modification(redValue, greenValue, blueValue, alphaValue);
+	return std::make_unique<swap_modification>(redValue, greenValue, blueValue, alphaValue);
 }
 
 } // end anon namespace
