@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -16,8 +16,8 @@
 #pragma once
 
 #include "config.hpp"
+#include "gui/sort_order.hpp"
 #include "terrain/translation.hpp"
-#include "utils/make_enum.hpp"
 
 #include <utility>
 
@@ -41,9 +41,15 @@ namespace preferences {
 	extern const int def_window_width;
 	extern const int def_window_height;
 
+	extern const int max_window_width;
+	extern const int max_window_height;
+
 	extern const int min_font_scaling;
 	extern const int max_font_scaling;
 
+	extern const int min_pixel_scale;
+	extern const int max_pixel_scale;
+
 	void write_preferences();
 
 	void set(const std::string& key, const std::string &value);
@@ -53,7 +59,7 @@ namespace preferences {
 	void set(const std::string& key, const config::attribute_value& value);
 	void clear(const std::string& key);
 	void set_child(const std::string& key, const config& val);
-	const config &get_child(const std::string &key);
+	optional_const_config get_child(const std::string &key);
 	std::string get(const std::string& key);
 	std::string get(const std::string& key, const std::string& def);
 	bool get(const std::string& key, bool def);
@@ -75,17 +81,26 @@ namespace preferences {
 	point resolution();
 	void _set_resolution(const point& res);
 
+	int pixel_scale();
+	void set_pixel_scale(const int scale);
+
+	bool auto_pixel_scale();
+	void set_auto_pixel_scale(bool choice);
+
 	bool maximized();
 	void _set_maximized(bool ison);
 
 	bool fullscreen();
 	void _set_fullscreen(bool ison);
 
+	bool vsync();
+	void set_vsync(bool ison);
+
 	bool turbo();
-	void _set_turbo(bool ison);
+	void set_turbo(bool ison);
 
 	double turbo_speed();
-	void save_turbo_speed(const double speed);
+	void set_turbo_speed(const double speed);
 
 	int font_scaling();
 	void set_font_scaling(int scale);
@@ -95,10 +110,10 @@ namespace preferences {
 	void keepalive_timeout(int seconds);
 
 	bool idle_anim();
-	void _set_idle_anim(const bool ison);
+	void set_idle_anim(const bool ison);
 
 	int idle_anim_rate();
-	void _set_idle_anim_rate(const int rate);
+	void set_idle_anim_rate(int rate);
 
 	std::string language();
 	void set_language(const std::string& s);
@@ -152,7 +167,7 @@ namespace preferences {
 	void clear_hotkeys();
 
 	void add_alias(const std::string& alias, const std::string& command);
-	const config &get_alias();
+	optional_const_config get_alias();
 
 
 	std::string allied_color();
@@ -170,9 +185,6 @@ namespace preferences {
 	std::string moved_color();
 	void set_moved_color(const std::string& color_id);
 
-	std::string disengaged_color();
-	void set_disengaged_color(const std::string& color_id);
-
 	bool show_ally_orb();
 	void set_show_ally_orb(bool show_orb);
 
@@ -243,7 +255,7 @@ namespace preferences {
 	void set_ellipses(bool ison);
 
 	bool grid();
-	void _set_grid(bool ison);
+	void set_grid(bool ison);
 
 	bool confirm_load_save_from_different_version();
 
@@ -259,13 +271,69 @@ namespace preferences {
 	std::string addon_manager_saved_order_name();
 	void set_addon_manager_saved_order_name(const std::string& value);
 
-	// Sorting for GUI2 listboxes
-	MAKE_ENUM(SORT_ORDER,
-		(NONE, "none")
-		(ASCENDING, "ascending")
-		(DESCENDING, "descending")
-	)
-	SORT_ORDER addon_manager_saved_order_direction();
-	void set_addon_manager_saved_order_direction(SORT_ORDER value);
+	sort_order::type addon_manager_saved_order_direction();
+	void set_addon_manager_saved_order_direction(sort_order::type value);
+
+	std::string selected_achievement_group();
+	void set_selected_achievement_group(const std::string& content_for);
+
+	/**
+	 * @param content_for The achievement group the achievement is part of.
+	 * @param id The ID of the achievement within the achievement group.
+	 * @return True if the achievement exists and is completed, false otherwise.
+	 */
+	bool achievement(const std::string& content_for, const std::string& id);
+	/**
+	 * Marks the specified achievement as completed.
+	 *
+	 * @param content_for The achievement group the achievement is part of.
+	 * @param id The ID of the achievement within the achievement group.
+	 */
+	void set_achievement(const std::string& content_for, const std::string& id);
+
+	/**
+	 * Increments the achievement's current progress by @a amount if it hasn't already been completed.
+	 * If you only want to check the achievement's current progress, then omit the last three arguments.
+	 * @a amount defaults to 0, which will result in the current progress value being returned without being changed (x + 0 == x).
+	 *
+	 * Note that this uses the same [in_progress] as is used for set_sub_achievement().
+	 *
+	 * @param content_for The id of the achievement group this achievement is in.
+	 * @param id The id for the specific achievement in the achievement group.
+	 * @param limit The maximum value that a specific call to this function can increase the achievement progress value.
+	 * @param max_progress The value when the achievement is considered completed.
+	 * @param amount The amount to progress the achievement.
+	 * @return The achievement's current progress, or -1 if it has already been completed.
+	 */
+	int progress_achievement(const std::string& content_for, const std::string& id, int limit = 999999, int max_progress = 999999, int amount = 0);
+
+	/**
+	 * @param content_for The achievement group the achievement is part of.
+	 * @param id The ID of the achievement within the achievement group.
+	 * @param sub_id The ID of the sub-achievement within the achievement.
+	 * @return True if the sub-achievement exists and is completed, false otherwise.
+	 */
+	bool sub_achievement(const std::string& content_for, const std::string& id, const std::string& sub_id);
+
+	/**
+	 * Marks the specified sub-achievement as completed.
+	 *
+	 * Note that this uses the same [in_progress] as is used for progress_achievement().
+	 *
+	 * @param content_for The achievement group the achievement is part of.
+	 * @param id The ID of the achievement within the achievement group.
+	 * @param sub_id The ID of the sub-achievement within the achievement.
+	 */
+	void set_sub_achievement(const std::string& content_for, const std::string& id, const std::string& sub_id);
+
+	/**
+	 * @param addon_id The chosen addon id from the editor to store in the preferences.
+	 */
+	void set_editor_chosen_addon(const std::string& addon_id);
+
+	/**
+	 * @return The most recently selected add-on id from the editor. May be an empty string.
+	 */
+	std::string editor_chosen_addon();
 
 } // end namespace preferences
