@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2021
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -46,39 +46,39 @@ public:
 	class context
 	{
 	public:
-		context(const display_context & dc, display & disp, const tod_manager & tod, std::shared_ptr<wb::manager> wb, utils::optional_reference<events::mouse_handler> mhb) : dc_(dc), disp_(disp), tod_(tod), wb_(wb), mhb_(mhb) {}
+		context(const display_context & dc, const display & disp, const tod_manager & tod, std::shared_ptr<wb::manager> wb, utils::optional_reference<events::mouse_handler> mhb) : dc_(dc), disp_(disp), tod_(tod), wb_(wb), mhb_(mhb) {}
 
 		const std::vector<team> & teams() const { return dc_.teams(); }
 		const unit_map & units() const { return dc_.units(); }
 		const gamemap & map() const { return dc_.map(); }
 
 		const display_context & dc() const { return dc_; }
-		display & screen() { return disp_; }
+		const display & screen() const { return disp_; }
 		const tod_manager & tod() const { return tod_; }
-		std::shared_ptr<wb::manager> wb() { return wb_; }
-		utils::optional_reference<events::mouse_handler> mhb() { return mhb_; }
+		std::shared_ptr<wb::manager> wb() const { return wb_; }
+		utils::optional_reference<events::mouse_handler> mhb() const { return mhb_; }
 
 	private:
-		const display_context & dc_;
-		display & disp_;
-		const tod_manager & tod_;
+		const display_context& dc_;
+		const display& disp_;
+		const tod_manager& tod_;
 		std::shared_ptr<wb::manager> wb_;
 		utils::optional_reference<events::mouse_handler> mhb_;
 	};
 
 	struct generator
 	{
-		virtual config generate(context & ct) = 0;
+		virtual config generate(const context& ct) = 0;
 		virtual ~generator() {}
 	};
 
 	void register_generator(const std::string &name, generator *);
 
-	config generate_report(const std::string &name, context & ct, bool only_static = false);
+	config generate_report(const std::string &name, const context& ct, bool only_static = false);
 
 	const std::set<std::string> &report_list();
 
-	using generator_function = std::function<config(reports::context&)>;
+	using generator_function = std::function<config(const reports::context&)>;
 
 	typedef std::map<std::string, std::shared_ptr<reports::generator>> dynamic_report_generators;
 
