@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by Thomas Baumhauer <thomas.baumhauer@NOSPAMgmail.com>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -129,8 +129,11 @@ public:
 	 * @param player The player iterator used to communicate with the player's client.
 	 * @param player_id The forum ID of the player to get the game history for.
 	 * @param offset Where to start returning rows to the client from the query results.
+	 * @param search_game_name Query for games matching this name. Supports leading and/or trailing wildcards.
+	 * @param search_content_type The content type to query for (ie: scenario)
+	 * @param search_content Query for games using this content ID. Supports leading and/or trailing wildcards.
 	 */
-	void async_get_and_send_game_history(boost::asio::io_service& io_service, wesnothd::server& s, wesnothd::player_iterator player, int player_id, int offset);
+	void async_get_and_send_game_history(boost::asio::io_service& io_service, wesnothd::server& s, wesnothd::player_iterator player, int player_id, int offset, std::string& search_game_name, int search_content_type, std::string& search_content);
 
 	/**
 	 * Inserts game related information.
@@ -179,8 +182,8 @@ public:
 	 * @param type The add-on content's type (ie: era, scenario, etc).
 	 * @param name The name of the content.
 	 * @param id The id of the content.
-	 * @param addon_id The source add-on for the content.
-	 * @param addon_version The version of the source add-on.
+	 * @param addon_id The id of the addon that the content is from.
+	 * @param addon_version The version of the add-on.
 	 * @return The number of rows inserted which should always be 1.
 	 */
 	unsigned long long db_insert_game_content_info(const std::string& uuid, int game_id, const std::string& type, const std::string& name, const std::string& id, const std::string& addon_id, const std::string& addon_version);
@@ -220,8 +223,9 @@ public:
 	 * @param version The add-on's version from the pbl.
 	 * @param forum_auth Whether the provided author and password should be matched a forum account or not.
 	 * @param topic_id The forum topic ID of the add-on's feedback thread, 0 if not present.
+	 * @param uploader The person uploading this version of the add-on.
 	 */
-	void db_insert_addon_info(const std::string& instance_version, const std::string& id, const std::string& name, const std::string& type, const std::string& version, bool forum_auth, int topic_id);
+	void db_insert_addon_info(const std::string& instance_version, const std::string& id, const std::string& name, const std::string& type, const std::string& version, bool forum_auth, int topic_id, const std::string uploader);
 
 	/**
 	 * Inserts into the database for when a player logs in.
@@ -258,6 +262,68 @@ public:
 	 */
 	void get_ips_for_user(const std::string& username, std::ostringstream* out);
 
+	/**
+	 * @param user The player's username.
+	 * @return The player's email address from the phpbb forum database.
+	 */
+	std::string get_user_email(const std::string& user);
+
+	/**
+	 * Increments the download count for this add-on for the specific version.
+	 *
+	 * @param instance_version The version of campaignd the add-on was uploaded to.
+	 * @param id The add-on's ID (aka directory name).
+	 * @param version The version of the add-on being downloaded. May not be the most recent version.
+	 */
+	void db_update_addon_download_count(const std::string& instance_version, const std::string& id, const std::string& version);
+
+	/**
+	 * Checks whether the provided username is the primary author of the add-on.
+	 *
+	 * @param instance_version Which major version this is for (1.16, 1.17, etc).
+	 * @param id The ID of the add-on.
+	 * @param username The username attempting to do something with the add-on.
+	 * @return true if the user is the primary author of an addon, false otherwise.
+	 */
+	bool db_is_user_primary_author(const std::string& instance_version, const std::string& id, const std::string& username);
+
+	/**
+	 * Checks whether the provided username is a secondary author of the add-on.
+	 *
+	 * @param instance_version Which major version this is for (1.16, 1.17, etc).
+	 * @param id The ID of the add-on.
+	 * @param username The username attempting to do something with the add-on.
+	 * @return true if the user is a secondary author of an addon, false otherwise.
+	 */
+	bool db_is_user_secondary_author(const std::string& instance_version, const std::string& id, const std::string& username);
+
+	/**
+	 * Removes the authors information from addon_author for a particular addon and version.
+	 *
+	 * @param instance_version Which major version this is for (1.16, 1.17, etc).
+	 * @param id The ID of the add-on.
+	 */
+	void db_delete_addon_authors(const std::string& instance_version, const std::string& id);
+
+	/**
+	 * Inserts rows for the primary and secondary authors for a particular addon and version.
+	 *
+	 * @param instance_version Which major version this is for (1.16, 1.17, etc).
+	 * @param id The ID of the add-on.
+	 * @param primary_author The primary author of the add-on.
+	 * @param secondary_authors The secondary authors of the add-on.
+	 */
+	void db_insert_addon_authors(const std::string& instance_version, const std::string& id, const std::string& primary_author, const std::vector<std::string>& secondary_authors);
+
+	/**
+	 * Checks whether any author information exists for a particular addon and version, since if there's no author information then of course no primary or secondary authors will ever be found.
+	 *
+	 * @param instance_version Which major version this is for (1.16, 1.17, etc).
+	 * @param id The ID of the add-on.
+	 * @return true if any author information exists for this addon, false otherwise.
+	 */
+	bool db_do_any_authors_exist(const std::string& instance_version, const std::string& id);
+
 private:
 	/** An instance of the class responsible for executing the queries and handling the database connection. */
 	dbconn conn_;
@@ -285,11 +351,4 @@ private:
 	 * @return The player's forum registration date.
 	 */
 	std::time_t get_registrationdate(const std::string& user);
-
-	/**
-	 * @param name The player's username.
-	 * @param group_id The forum group ID to check if the user is part of.
-	 * @return Whether the user is a member of the forum group.
-	 */
-	bool is_user_in_group(const std::string& name, int group_id);
 };
