@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2008 - 2022
+	Copyright (C) 2008 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -13,7 +13,6 @@
 	See the COPYING file for more details.
 */
 
-#include <iostream>
 #include <sstream>
 
 #include <boost/iostreams/copy.hpp>
@@ -25,9 +24,11 @@
 #include "server/common/simple_wml.hpp"
 
 #include "log.hpp"
+#include "utils/general.hpp"
 
 static lg::log_domain log_config("config");
 #define ERR_SWML LOG_STREAM(err, log_config)
+#define LOG_SWML LOG_STREAM(info, log_config)
 
 namespace simple_wml {
 
@@ -95,7 +96,7 @@ char* uncompress_buffer(const string_span& input, string_span* span)
 	} catch (const std::bad_alloc& e) {
 		ERR_SWML << "ERROR: bad_alloc caught in uncompress_buffer() state "
 		<< state << " alloc bytes " << nalloc << " with input: '"
-		<< input << "' " << e.what() << std::endl;
+		<< input << "' " << e.what();
 		throw error("Bad allocation request in uncompress_buffer().");
 	}
 }
@@ -148,7 +149,7 @@ char* compress_buffer(const char* input, string_span* span, bool bzip2)
 	} catch (const std::bad_alloc& e) {
 		ERR_SWML << "ERROR: bad_alloc caught in compress_buffer() state "
 		<< state << " alloc bytes " << nalloc << " with input: '"
-		<< input << "' " << e.what() << std::endl;
+		<< input << "' " << e.what();
 		throw error("Bad allocation request in compress_buffer().");
 	}
 }
@@ -195,7 +196,7 @@ char* string_span::duplicate() const
 error::error(const char* msg)
   : game::error(msg)
 {
-	ERR_SWML << "ERROR: '" << msg << "'" << std::endl;
+	ERR_SWML << "ERROR: '" << msg << "'";
 }
 
 std::ostream& operator<<(std::ostream& o, const string_span& s)
@@ -278,7 +279,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 		default: {
 			const char* end = strchr(s, '=');
 			if(end == nullptr) {
-				ERR_SWML << "attribute: " << s << std::endl;
+				ERR_SWML << "attribute: " << s;
 				throw error("did not find '=' after attribute");
 			}
 
@@ -294,7 +295,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 			if (*s != '"') {
 				end = strchr(s, '\n');
 				if (!end) {
-					ERR_SWML << "ATTR: '" << name << "' (((" << s << ")))" << std::endl;
+					ERR_SWML << "ATTR: '" << name << "' (((" << s << ")))";
 					throw error("did not find end of attribute");
 				}
 				if (memchr(s, '"', end - s))
@@ -346,7 +347,7 @@ node::node(document& doc, node* parent, const char** str, int depth) :
 			read_attribute:
 			string_span value(s, end - s);
 			if(attr_.empty() == false && !(attr_.back().key < name)) {
-				ERR_SWML << "attributes: '" << attr_.back().key << "' < '" << name << "'" << std::endl;
+				ERR_SWML << "attributes: '" << attr_.back().key << "' < '" << name << "'";
 				throw error("attributes not in order");
 			}
 
@@ -996,6 +997,7 @@ document::document(string_span compressed_buf) :
 	try {
 		root_ = new node(*this, nullptr, &cbuf);
 	} catch(...) {
+		ERR_SWML << "Caught exception creating a new simple_wml node: " << utils::get_unknown_exception_type();
 		delete [] buffers_.front();
 		buffers_.clear();
 		throw;
@@ -1050,7 +1052,7 @@ const char* document::output()
 		buf = new char[buf_size];
 	} catch (const std::bad_alloc& e) {
 		ERR_SWML << "ERROR: Trying to allocate " << buf_size << " bytes. "
-		<< e.what() << std::endl;
+		<< e.what();
 		throw error("Bad allocation request in output().");
 	}
 	buffers_.push_back(buf);
@@ -1238,32 +1240,3 @@ void swap(document& lhs, document& rhs)
 }
 
 }
-
-#ifdef UNIT_TEST_SIMPLE_WML
-
-int main(int argc, char** argv)
-{
-	char* doctext = strdup(
-"[test]\n"
-"a=\"blah\"\n"
-"b=\"blah\"\n"
-"c=\"\\\\\"\n"
-"d=\"\\\"\"\n"
-"[/test]");
-	std::cerr << doctext << "\n";
-	simple_wml::document doc(doctext);
-
-	simple_wml::node& node = doc.root();
-	simple_wml::node* test_node = node.child("test");
-	assert(test_node);
-	assert((*test_node)["a"] == "blah");
-	assert((*test_node)["b"] == "blah");
-	assert((*test_node)["c"] == "\\\\");
-	assert((*test_node)["d"] == "\\\"");
-
-	node.set_attr("blah", "blah");
-	test_node->set_attr("e", "f");
-	std::cerr << doc.output();
-}
-
-#endif
