@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2014 - 2022
+	Copyright (C) 2014 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -48,7 +48,7 @@ ignored_checkup::~ignored_checkup()
 bool ignored_checkup::local_checkup(const config& /*expected_data*/, config& real_data)
 {
 	assert(real_data.empty());
-	LOG_REPLAY << "ignored_checkup::local_checkup called\n";
+	LOG_REPLAY << "ignored_checkup::local_checkup called";
 	return true;
 }
 
@@ -67,7 +67,7 @@ bool synced_checkup::local_checkup(const config& expected_data, config& real_dat
 	if(buffer_.child_count("result") > pos_)
 	{
 		//copying objects :o
-		real_data = buffer_.child("result",pos_);
+		real_data = buffer_.mandatory_child("result",pos_);
 		pos_ ++;
 		return real_data == expected_data;
 	}
