@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2003 - 2022
+	Copyright (C) 2003 - 2024
 	by David White <dave@whitevine.net>
 	Part of the Battle for Wesnoth Project https://www.wesnoth.org/
 
@@ -23,6 +23,7 @@
 #include "color.hpp"
 #include "config.hpp"
 #include "generic_event.hpp"
+#include "sdl/rect.hpp"
 
 #include <memory>
 #include <SDL2/SDL_rect.h>
@@ -48,8 +49,8 @@ class theme
 		object(std::size_t sw, std::size_t sh, const config& cfg);
 		virtual ~object() { }
 
-		virtual SDL_Rect& location(const SDL_Rect& screen) const;
-		const SDL_Rect& get_location() const { return loc_; }
+		virtual rect& location(const SDL_Rect& screen) const;
+		const rect& get_location() const { return loc_; }
 		const std::string& get_id() const { return id_; }
 
 		// This supports relocating of theme elements ingame.
@@ -70,9 +71,9 @@ class theme
 	private:
 		bool location_modified_;
 		std::string id_;
-		SDL_Rect loc_;
-		mutable SDL_Rect relative_loc_;
-		mutable SDL_Rect last_screen_;
+		rect loc_;
+		mutable rect relative_loc_;
+		mutable rect last_screen_;
 
 		ANCHORING xanchor_, yanchor_;
 		std::size_t spec_width_, spec_height_;
@@ -269,18 +270,15 @@ public:
 	const menu *get_menu_item(const std::string &key) const;
 	const action* get_action_item(const std::string &key) const;
 
-	const SDL_Rect& main_map_location(const SDL_Rect& screen) const
+	const rect& main_map_location(const SDL_Rect& screen) const
 		{ return main_map_.location(screen); }
-	const SDL_Rect& mini_map_location(const SDL_Rect& screen) const
+	const rect& mini_map_location(const SDL_Rect& screen) const
 		{ return mini_map_.location(screen); }
-	const SDL_Rect& unit_image_location(const SDL_Rect& screen) const
+	const rect& unit_image_location(const SDL_Rect& screen) const
 		{ return unit_image_.location(screen); }
-	const SDL_Rect& palette_location(const SDL_Rect& screen) const
+	const rect& palette_location(const SDL_Rect& screen) const
 		{ return palette_.location(screen); }
 
-    static void set_known_themes(const game_config_view* cfg);
-    static std::vector<theme_info> get_known_themes();
-
 	const border_t& border() const { return border_; }
 
 	events::generic_event& theme_reset_event() { return theme_reset_event_; }
@@ -295,7 +293,6 @@ private:
 	//atm this is used for replay_controller to add replay controls to the standard theme
 	events::generic_event theme_reset_event_;
 
-	static std::map<std::string, config> known_themes;
 	std::string cur_theme;
 	config cfg_;
 	std::vector<panel> panels_;
@@ -315,4 +312,16 @@ private:
 
 	SDL_Rect screen_dimensions_;
 	std::size_t cur_spec_width_, cur_spec_height_;
+
+	static inline std::map<std::string, config> known_themes{};
+
+public:
+	/** Copies the theme configs from the main game config. */
+	static void set_known_themes(const game_config_view* cfg);
+
+	/** Returns the saved config for the theme with the given ID. */
+	static const config& get_theme_config(const std::string& id);
+
+	/** Returns minimal info about saved themes, optionally including hidden ones. */
+	static std::vector<theme_info> get_basic_theme_info(bool include_hidden = false);
 };
